import warnings
from firewxpy.calc import scaling
warnings.filterwarnings('ignore')

def hawaiian_islands_coords(island):

    if island == None:
        western_bound, eastern_bound, southern_bound, northern_bound = -160.3, -154.73, 18.76, 22.28
        decimate = scaling.get_NDFD_decimation_by_state('hi')
    if island == 'Oahu' or island == 'oahu':
        western_bound, eastern_bound, southern_bound, northern_bound = -158.31, -157.62, 21.22, 21.75
        decimate = 75
    if island == 'Hawaii' or island == 'hawaii':
        western_bound, eastern_bound, southern_bound, northern_bound = -156.21, -154.70, 18.83, 20.35
        decimate = 50
    if island == 'Maui' or island == 'maui':
        western_bound, eastern_bound, southern_bound, northern_bound = -156.76, -155.90, 20.37, 21.08
        decimate = 5
    if island == 'Kauai' or island == 'kauai':
        western_bound, eastern_bound, southern_bound, northern_bound = -159.84, -159.25, 21.84, 22.25
        decimate = 5
    if island == 'Molokai' or island == 'molokai':
        western_bound, eastern_bound, southern_bound, northern_bound = -157.5, -156.6, 21.02, 21.24
        decimate = 67
    if island == 'Lanai' or island == 'lanai':
        western_bound, eastern_bound, southern_bound, northern_bound = -157.07, -156.79, 20.71, 20.95
        decimate = 35
    if island == 'Niihau' or island == 'niihau':
        western_bound, eastern_bound, southern_bound, northern_bound = -160.5, -159.75, 21.7, 22.05
        decimate = 5
    

    return western_bound, eastern_bound, southern_bound, northern_bound, decimate

def get_metar_mask(state, gacc_region, rtma_ws=False):

    r'''
    This function returns the value for the METAR mask for a given state and/or gacc region. 

    Required Arguments: 1) state (String) - The two letter state abbreviation for the state the user wishes to make the graphic for. 
                            If the user wishes to make a graphic for the entire CONUS, there are 4 acceptable abbreviations: 'US' or 'us'
                            or 'USA' or 'usa'. Example: If the user wishes to make a plot for the state of California both 'CA' or 'ca' are
                            acceptable. Default setting is 'us'. If the user wishes to make a plot based on gacc_region, this value must be 
                            changed to None. 

                        2) gacc_region (String) - The abbreviation for each of the 10 GACC regions. Default setting is None. 
                            If the user wishes to make a plot based on GACC Region than state, the state variable must be set to 
                            None and the gacc_region variable must be set to one of the acceptable abbreviations. 

                            Here is a list of acceptable gacc_region abbreviations:

                            South Ops: 'OSCC' or 'oscc' or 'SOPS' or 'sops'
                            North Ops: 'ONCC' or 'oncc' or 'NOPS' or 'nops'
                            Great Basin: 'GBCC' or 'gbcc' or 'GB' or 'gb'
                            Northern Rockies: 'NRCC' or 'nrcc' or 'NR' or 'nr'
                            Rocky Mountain: 'RMCC' or 'rmcc' or 'RM' or 'rm'
                            Southwest: 'SWCC' or 'swcc' or 'SW' or 'sw'
                            Southern: 'SACC' or 'sacc' or 'SE' or 'se'
                            Eastern: 'EACC' or 'eacc' or 'E' or 'e'
                            Pacific Northwest: 'PNW' or 'pnw' or 'NWCC' or 'nwcc' or 'NW' or 'nw'
                            Alaska: Setting state='AK' or state='ak' suffices here. Leave gacc_region=None and set the state variable as shown. 

    Optional Arguments: rtma_ws (Boolean) - Default = False. For full METAR plots, set this to False to allow for more spacing between observation
                        sites to allow for the size of the full station plot. For plots where the user only plots the wind barbs of the observed winds,
                        this value is to be set to True to allow for more data points since the wind barbs do not need as much spacing as a full METAR station plot.

    Returns: The value for the mask of the METAR locations. 

    '''

    if rtma_ws == False:
        if state != None and gacc_region == None:
            
            if state == 'CONUS' or state == 'conus':
                mask = 250000
            if state == 'CA' or state == 'ca':
                mask = 80000
            if state == 'FL' or state == 'fl':
                mask = 60000
            if state == 'ME' or state == 'me':
                mask = 30000
            if state == 'NH' or state == 'nh':
                mask = 30000
            if state == 'VT' or state == 'vt':
                mask = 30000
            if state == 'MA' or state == 'ma':
                mask = 30000
            if state == 'RI' or state == 'ri':
                mask = 30000
            if state == 'NY' or state == 'ny':
                mask = 30000
            if state == 'CT' or state == 'ct':
                mask = 30000
            if state == 'NJ' or state == 'nj':
                mask = 30000
            if state == 'DE' or state == 'de':
                mask = 30000
            if state == 'PA' or state == 'pa':
                mask = 30000
            if state == 'OH' or state == 'oh':
                mask = 30000
            if state == 'MI' or state == 'mi':
                mask = 50000
            if state == 'MN' or state == 'mn':
                mask = 50000
            if state == 'WI' or state == 'wi':
                mask = 30000
            if state == 'IA' or state == 'ia':
                mask = 30000
            if state == 'IN' or state == 'in':
                mask = 30000
            if state == 'MO' or state == 'mo':
                mask = 30000
            if state == 'IL' or state == 'il':
                mask = 30000
            if state == 'ND' or state == 'nd':
                mask = 30000
            if state == 'SD' or state == 'sd':
                mask = 30000
            if state == 'NE' or state == 'ne':
                mask = 30000
            if state == 'MD' or state == 'md':
                mask = 30000
            if state == 'VA' or state == 'va':
                mask = 30000
            if state == 'SC' or state == 'sc':
                mask = 30000
            if state == 'KY' or state == 'ky':
                mask = 30000
            if state == 'WV' or state == 'wv':
                mask = 30000
            if state == 'NC' or state == 'nc':
                mask = 40000
            if state == 'NV' or state == 'nv':
                mask = 30000
            if state == 'FL' or state == 'fl':
                mask = 30000
            if state == 'OR' or state == 'or':
                mask = 30000
            if state == 'WA' or state == 'wa':
                mask = 30000 
            if state == 'ID' or state == 'id':
                mask = 30000
            if state == 'GA' or state == 'ga':
                mask = 30000
            if state == 'AL' or state == 'al':
                mask = 30000
            if state == 'MS' or state == 'ms':
                mask = 30000
            if state == 'LA' or state == 'la':
                mask = 30000
            if state == 'AR' or state == 'ar':
                mask = 30000
            if state == 'TX' or state == 'tx':
                mask = 70000
            if state == 'OK' or state == 'ok':
                mask = 30000 
            if state == 'NM' or state == 'nm':
                mask = 30000
            if state == 'AZ' or state == 'az':
                mask = 30000 
            if state == 'UT' or state == 'ut':
                mask = 30000 
            if state == 'CO' or state == 'co':
                mask = 30000
            if state == 'WY' or state == 'wy':
                mask = 30000 
            if state == 'MT' or state == 'mt':
                mask = 30000
            if state == 'KS' or state == 'ks':
                mask = 30000 
            if state == 'TN' or state == 'tn':
                mask = 30000

        if state == None and gacc_region != None:
            if gacc_region == 'OSCC' or gacc_region == 'oscc' or gacc_region == 'SOPS' or gacc_region == 'sops':
                mask = 40000
            if gacc_region == 'ONCC' or gacc_region == 'oncc' or gacc_region == 'NOPS' or gacc_region == 'nops':
                mask = 40000
            if gacc_region == 'GBCC' or gacc_region == 'gbcc' or gacc_region == 'GB' or gacc_region == 'gb':
                mask = 70000
            if gacc_region == 'NRCC' or gacc_region == 'nrcc' or gacc_region == 'NR' or gacc_region == 'nr':
                mask = 70000
            if gacc_region == 'RMCC' or gacc_region == 'rmcc' or gacc_region == 'RM' or gacc_region == 'rm':
                mask = 60000
            if gacc_region == 'SWCC' or gacc_region == 'swcc' or gacc_region == 'SW' or gacc_region == 'sw':
                mask = 30000
            if gacc_region == 'SACC' or gacc_region == 'sacc' or gacc_region == 'SE' or gacc_region == 'se':
                mask = 160000
            if gacc_region == 'EACC' or gacc_region == 'eacc' or gacc_region == 'E' or gacc_region == 'e':
                mask = 160000
            if gacc_region == 'PNW' or gacc_region == 'pnw' or gacc_region == 'NWCC' or gacc_region == 'nwcc' or gacc_region == 'NW' or gacc_region == 'nw':
                mask = 30000

    if rtma_ws == True:
        if state != None and gacc_region == None:
            
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                mask = 100000
            if state == 'CA' or state == 'ca':
                mask = 300
            if state == 'FL' or state == 'fl':
                mask = 300
            if state == 'GA' or state == 'ga':
                mask = 300
            if state == 'TN' or state == 'tn':
                mask = 300
            if state == 'KY' or state == 'ky':
                mask = 300    
            if state == 'ME' or state == 'me':
                mask = 10    
            if state == 'NH' or state == 'nh':
                mask = 10    
            if state == 'VT' or state == 'vt':
                mask = 10    
            if state == 'MA' or state == 'ma':
                mask = 10  
            if state == 'RI' or state == 'ri':
                mask = 10  
            if state == 'CT' or state == 'ct':
                mask = 10  
            if state == 'NJ' or state == 'nj':
                mask = 10  
            if state == 'DE' or state == 'de':
                mask = 10  
            if state == 'PA' or state == 'pa':
                mask = 10  
            if state == 'OH' or state == 'oh':
                mask = 10  
            if state == 'MI' or state == 'mi':
                mask = 10  
            if state == 'MN' or state == 'mn':
                mask = 10  
            if state == 'WI' or state == 'wi':
                mask = 10  
            if state == 'IA' or state == 'ia':
                mask = 10  
            if state == 'IN' or state == 'in':
                mask = 10 
            if state == 'MO' or state == 'mo':
                mask = 10  
            if state == 'IL' or state == 'il':
                mask = 10  
            if state == 'ND' or state == 'nd':
                mask = 10 
            if state == 'SD' or state == 'sd':
                mask = 10 
            if state == 'NE' or state == 'ne':
                mask = 10 
            if state == 'MD' or state == 'md':
                mask = 10 
            if state == 'VA' or state == 'va':
                mask = 10 
            if state == 'SC' or state == 'sc':
                mask = 10 
            if state == 'KY' or state == 'ky':
                mask = 10 
            if state == 'WV' or state == 'wv':
                mask = 10 
            if state == 'NV' or state == 'nv':
                mask = 10 
            if state == 'NC' or state == 'nc':
                mask = 10 
            if state == 'FL' or state == 'fl':
                mask = 10 
            if state == 'OR' or state == 'or':
                mask = 10 
            if state == 'WA' or state == 'wa':
                mask = 10 
            if state == 'ID' or state == 'id':
                mask = 10 
            if state == 'GA' or state == 'ga':
                mask = 10 
            if state == 'AL' or state == 'al':
                mask = 10 
            if state == 'MS' or state == 'ms':
                mask = 10 
            if state == 'LA' or state == 'la':
                mask = 10 
            if state == 'AR' or state == 'ar':
                mask = 10 
            if state == 'TX' or state == 'tx':
                mask = 10 
            if state == 'OK' or state == 'ok':
                mask = 10 
            if state == 'NM' or state == 'nm':
                mask = 10 
            if state == 'AZ' or state == 'az':
                mask = 10 
            if state == 'UT' or state == 'ut':
                mask = 10 
            if state == 'CO' or state == 'co':
                mask = 10 
            if state == 'WY' or state == 'wy':
                mask = 10 
            if state == 'MT' or state == 'mt':
                mask = 10 
            if state == 'KS' or state == 'ks':
                mask = 10 
            if state == 'TN' or state == 'tn':
                mask = 10 
                
        if state == None and gacc_region != None:
            if gacc_region == 'OSCC' or gacc_region == 'oscc' or gacc_region == 'SOPS' or gacc_region == 'sops':
                mask = 10
            if gacc_region == 'ONCC' or gacc_region == 'oncc' or gacc_region == 'NOPS' or gacc_region == 'nops':
                mask = 10
            if gacc_region == 'GBCC' or gacc_region == 'gbcc' or gacc_region == 'GB' or gacc_region == 'gb':
                mask = 10
            if gacc_region == 'NRCC' or gacc_region == 'nrcc' or gacc_region == 'NR' or gacc_region == 'nr':
                mask = 10
            if gacc_region == 'RMCC' or gacc_region == 'rmcc' or gacc_region == 'RM' or gacc_region == 'rm':
                mask = 10
            if gacc_region == 'SWCC' or gacc_region == 'swcc' or gacc_region == 'SW' or gacc_region == 'sw':
                mask = 10
            if gacc_region == 'SACC' or gacc_region == 'sacc' or gacc_region == 'SE' or gacc_region == 'se':
                mask = 60000
            if gacc_region == 'EACC' or gacc_region == 'eacc' or gacc_region == 'E' or gacc_region == 'e':
                mask = 60000
            if gacc_region == 'PNW' or gacc_region == 'pnw' or gacc_region == 'NWCC' or gacc_region == 'nwcc' or gacc_region == 'NW' or gacc_region == 'nw':
                mask = 10

    return mask

def get_quiver_dims(state, gacc_region):


    if state != None and gacc_region == None:

        if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':

            minshaft=0.000005 
            headlength=5 
            headwidth=3

        else:

            minshaft=0.000000000000000000000000005  
            headlength=20 
            headwidth=17

    if state == None and gacc_region != None:

            minshaft=0.000000000000000000000000005  
            headlength=20 
            headwidth=17
        
    return minshaft, headlength, headwidth

def get_label_coords(state, gacc_region):

    if state != None and gacc_region == None:

        if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':

            x_coord = 0.91
            y_coord = 0.92

        if state == 'CA' or state == 'ca':

            x_coord = 0.8
            y_coord = 0.94

        if state == 'ME' or state == 'me':

            x_coord = 0.77
            y_coord = 0.93

        if state == 'NH' or state == 'nh':

            x_coord = 0.77
            y_coord = 0.94

        if state == 'VT' or state == 'vt':

            x_coord = 0.77
            y_coord = 0.02

        if state == 'MA' or state == 'ma':

            x_coord = 0.83
            y_coord = 0.9

        if state == 'RI' or state == 'ri':

            x_coord = 0.78
            y_coord = 0.93

        if state == 'CT' or state == 'ct':

            x_coord = 0.78
            y_coord = 0.02

        if state == 'NJ' or state == 'nj':

            x_coord = 0.73
            y_coord = 0.95

        if state == 'PA' or state == 'pa':

            x_coord = 0.85
            y_coord = 0.92

        if state == 'OH' or state == 'oh':

            x_coord = 0.82
            y_coord = 0.02

        if state == 'MI' or state == 'mi':

            x_coord = 0.82
            y_coord = 0.94

        if state == 'MN' or state == 'mn':

            x_coord = 0.85
            y_coord = 0.93

        if state == 'WI' or state == 'wi':

            x_coord = 0.85
            y_coord = 0.94

        if state == 'IA' or state == 'ia':

            x_coord = 0.88
            y_coord = 0.92

        if state == 'IN' or state == 'in':

            x_coord = 0.75
            y_coord = 0.02

        if state == 'MO' or state == 'mo':

            x_coord = 0.88
            y_coord = 0.94

        if state == 'IL' or state == 'il':

            x_coord = 0.01
            y_coord = 0.93

        if state == 'NY' or state == 'ny':

            x_coord = 0.87
            y_coord = 0.93

        if state == 'FL' or state == 'fl':

            x_coord = 0.86
            y_coord = 0.94

        if state == 'ND' or state == 'nd':

            x_coord = 0.88
            y_coord = 0.02

        if state == 'SD' or state == 'sd':

            x_coord = 0.88
            y_coord = 0.02

        if state == 'NE' or state == 'ne':

            x_coord = 0.87
            y_coord = 0.92

        if state == 'MD' or state == 'md':

            x_coord = 0.87
            y_coord = 0.92

        if state == 'VA' or state == 'va':

            x_coord = 0.87
            y_coord = 0.92

        if state == 'SC' or state == 'sc':

            x_coord = 0.86
            y_coord = 0.94

        if state == 'KY' or state == 'ky':

            x_coord = 0.87
            y_coord = 0.92

        if state == 'WV' or state == 'wv':

            x_coord = 0.86
            y_coord = 0.94

        if state == 'NC' or state == 'nc':

            x_coord = 0.87
            y_coord = 0.93

        if state == 'NV' or state == 'nv':

            x_coord = 0.01
            y_coord = 0.15

        if state == 'OR' or state == 'or':

            x_coord = 0.86
            y_coord = 0.94

        if state == 'WA' or state == 'wa':

            x_coord = 0.88
            y_coord = 0.02

        if state == 'ID' or state == 'id':

            x_coord = 0.8
            y_coord = 0.94

        if state == 'GA' or state == 'ga':

            x_coord = 0.81
            y_coord = 0.94

        if state == 'AL' or state == 'al':

            x_coord = 0.75
            y_coord = 0.02

        if state == 'MS' or state == 'ms':

            x_coord = 0.01
            y_coord = 0.94

        if state == 'LA' or state == 'la':

            x_coord = 0.83
            y_coord = 0.94

        if state == 'TX' or state == 'tx':

            x_coord = 0.01
            y_coord = 0.94

        if state == 'OK' or state == 'ok':

            x_coord = 0.01
            y_coord = 0.18

        if state == 'NM' or state == 'nm':

            x_coord = 0.82
            y_coord = 0.02

        if state == 'AZ' or state == 'az':

            x_coord = 0.82
            y_coord = 0.02

        if state == 'UT' or state == 'ut':

            x_coord = 0.82
            y_coord = 0.94

        if state == 'CO' or state == 'co':

            x_coord = 0.85
            y_coord = 0.02

        if state == 'WY' or state == 'wy':

            x_coord = 0.85
            y_coord = 0.02

        if state == 'MT' or state == 'mt':

            x_coord = 0.85
            y_coord = 0.02

        if state == 'KS' or state == 'ks':

            x_coord = 0.85
            y_coord = 0.02

        if state == 'TN' or state == 'tn':

            x_coord = 0.85
            y_coord = 0.02

    if state == None and gacc_region != None:

        if gacc_region == 'OSCC' or gacc_region == 'oscc' or gacc_region == 'SOPS' or gacc_region == 'sops':

            x_coord = 0.85
            y_coord = 0.93   

        if gacc_region == 'ONCC' or gacc_region == 'oncc' or gacc_region == 'NOPS' or gacc_region == 'nops':

            x_coord = 0.01
            y_coord = 0.12   

        if gacc_region == 'GBCC' or gacc_region == 'gbcc' or gacc_region == 'GB' or gacc_region == 'gb':

            x_coord = 0.84
            y_coord = 0.93  

        if gacc_region == 'NRCC' or gacc_region == 'nrcc' or gacc_region == 'NR' or gacc_region == 'nr':

            x_coord = 0.88
            y_coord = 0.9 

        if gacc_region == 'RMCC' or gacc_region == 'rmcc' or gacc_region == 'RM' or gacc_region == 'rm':

            x_coord = 0.88
            y_coord = 0.9 

        if gacc_region == 'SWCC' or gacc_region == 'swcc' or gacc_region == 'SW' or gacc_region == 'sw':

            x_coord = 0.88
            y_coord = 0.9 

        if gacc_region == 'SACC' or gacc_region == 'sacc' or gacc_region == 'SE' or gacc_region == 'se':

            x_coord = 0.01
            y_coord = 0.15 

        if gacc_region == 'EACC' or gacc_region == 'eacc' or gacc_region == 'E' or gacc_region == 'e':

            x_coord = 0.88
            y_coord = 0.93

        if gacc_region == 'PNW' or gacc_region == 'pnw' or gacc_region == 'NWCC' or gacc_region == 'nwcc' or gacc_region == 'NW' or gacc_region == 'nw':

            x_coord = 0.85
            y_coord = 0.01
    
    return x_coord, y_coord

def get_colorbar_coords(state, gacc_region):

    if state != None and gacc_region == None:

        if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':

            x1, x2, y, x_size, fontsize = 0.18, 0.55, 0.25, 0.45, 12

        if state == 'AK' or state == 'ak':

            x1, x2, y, x_size, fontsize = 0.18, 0.55, 0.28, 0.45, 12

        if state == 'CA' or state == 'ca':

            x1, x2, y, x_size, fontsize = 0.19, 0.54, 0.05, 0.3, 12

        if state == 'ME' or state == 'me':

            x1, x2, y, x_size, fontsize = 0.19, 0.54, 0.05, 0.3, 12

        if state == 'NH' or state == 'nh':

            x1, x2, y, x_size, fontsize = 0.25, 0.53, 0.05, 0.25, 12

        if state == 'VT' or state == 'vt':

            x1, x2, y, x_size, fontsize = 0.25, 0.53, 0.05, 0.25, 12

        if state == 'MA' or state == 'ma':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.15, 0.35, 12

        if state == 'RI' or state == 'ri':

            x1, x2, y, x_size, fontsize = 0.2, 0.55, 0.05, 0.27, 12

        if state == 'CT' or state == 'ct':

            x1, x2, y, x_size, fontsize = 0.18, 0.55, 0.05, 0.3, 12

        if state == 'NJ' or state == 'nj':

            x1, x2, y, x_size, fontsize = 0.29, 0.58, 0.05, 0.15, 7

        if state == 'DE' or state == 'de':

            x1, x2, y, x_size, fontsize = 0.29, 0.58, 0.05, 0.15, 7

        if state == 'NY' or state == 'ny':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.15, 0.35, 12

        if state == 'PA' or state == 'pa':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.2, 0.35, 12

        if state == 'OH' or state == 'oh':

            x1, x2, y, x_size, fontsize = 0.19, 0.54, 0.05, 0.31, 12

        if state == 'MI' or state == 'mi':

            x1, x2, y, x_size, fontsize = 0.19, 0.54, 0.05, 0.33, 12

        if state == 'MN' or state == 'mn':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.15, 0.33, 12

        if state == 'WI' or state == 'wi':

            x1, x2, y, x_size, fontsize = 0.13, 0.54, 0.05, 0.36, 12

        if state == 'IA' or state == 'ia':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.2, 0.35, 12

        if state == 'IN' or state == 'in':

            x1, x2, y, x_size, fontsize = 0.29, 0.53, 0.05, 0.2, 7

        if state == 'MO' or state == 'mo':

            x1, x2, y, x_size, fontsize = 0.14, 0.54, 0.12, 0.36, 12

        if state == 'IL' or state == 'il':

            x1, x2, y, x_size, fontsize = 0.31, 0.53, 0.05, 0.2, 7

        if state == 'ND' or state == 'nd':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.2, 0.35, 12

        if state == 'SD' or state == 'sd':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.2, 0.35, 12

        if state == 'NE' or state == 'ne':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.24, 0.35, 12

        if state == 'MD' or state == 'md':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.24, 0.35, 12

        if state == 'VA' or state == 'va':

            x1, x2, y, x_size, fontsize = 0.14, 0.55, 0.24, 0.35, 12

        if state == 'SC' or state == 'sc':

            x1, x2, y, x_size, fontsize = 0.14, 0.54, 0.12, 0.36, 12

        if state == 'KY' or state == 'ky':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.24, 0.35, 12

        if state == 'WV' or state == 'wv':

            x1, x2, y, x_size, fontsize = 0.12, 0.52, 0.05, 0.36, 12

        if state == 'NC' or state == 'nc':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.24, 0.35, 12

        if state == 'NV' or state == 'nv':

            x1, x2, y, x_size, fontsize = 0.25, 0.53, 0.05, 0.25, 12

        if state == 'FL' or state == 'fl':

            x1, x2, y, x_size, fontsize = 0.14, 0.54, 0.07, 0.36, 12

        if state == 'OR' or state == 'or':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.2, 0.35, 12

        if state == 'WA' or state == 'wa':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.2, 0.35, 12

        if state == 'ID' or state == 'id':

            x1, x2, y, x_size, fontsize = 0.26, 0.52, 0.05, 0.23, 12

        if state == 'GA' or state == 'ga':

            x1, x2, y, x_size, fontsize = 0.25, 0.53, 0.07, 0.25, 12

        if state == 'AL' or state == 'al':

            x1, x2, y, x_size, fontsize = 0.28, 0.54, 0.07, 0.2, 12

        if state == 'MS' or state == 'ms':

            x1, x2, y, x_size, fontsize = 0.29, 0.53, 0.07, 0.18, 12

        if state == 'LA' or state == 'la':

            x1, x2, y, x_size, fontsize = 0.16, 0.53, 0.07, 0.33, 12

        if state == 'AR' or state == 'ar':

            x1, x2, y, x_size, fontsize = 0.16, 0.53, 0.07, 0.34, 12

        if state == 'TX' or state == 'tx':

            x1, x2, y, x_size, fontsize = 0.16, 0.53, 0.07, 0.34, 12

        if state == 'OK' or state == 'ok':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.25, 0.35, 12

        if state == 'NM' or state == 'nm':

            x1, x2, y, x_size, fontsize = 0.22, 0.53, 0.07, 0.25, 12

        if state == 'AZ' or state == 'az':

            x1, x2, y, x_size, fontsize = 0.22, 0.53, 0.07, 0.25, 12

        if state == 'UT' or state == 'ut':

            x1, x2, y, x_size, fontsize = 0.24, 0.53, 0.07, 0.26, 12

        if state == 'CO' or state == 'co':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.15, 0.35, 12

        if state == 'WY' or state == 'wy':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.15, 0.35, 12

        if state == 'MT' or state == 'mt':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.24, 0.35, 12

        if state == 'KS' or state == 'ks':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.24, 0.35, 12

        if state == 'TN' or state == 'tn':

            x1, x2, y, x_size, fontsize = 0.13, 0.55, 0.33, 0.35, 12

    if state == None and gacc_region != None:

        if gacc_region == 'OSCC' or gacc_region == 'oscc' or gacc_region == 'SOPS' or gacc_region == 'sops':

            x1, x2, y, x_size, fontsize = 0.13, 0.53, 0.07, 0.37, 12

        if gacc_region == 'ONCC' or gacc_region == 'oncc' or gacc_region == 'NOPS' or gacc_region == 'nops':

            x1, x2, y, x_size, fontsize = 0.22, 0.55, 0.07, 0.25, 12

        if gacc_region == 'GBCC' or gacc_region == 'gbcc' or gacc_region == 'GB' or gacc_region == 'gb':

            x1, x2, y, x_size, fontsize = 0.2, 0.53, 0.07, 0.3, 12

        if gacc_region == 'NRCC' or gacc_region == 'nrcc' or gacc_region == 'NR' or gacc_region == 'nr':

            x1, x2, y, x_size, fontsize = 0.13, 0.53, 0.19, 0.36, 12

        if gacc_region == 'RMCC' or gacc_region == 'rmcc' or gacc_region == 'RM' or gacc_region == 'rm':

            x1, x2, y, x_size, fontsize = 0.13, 0.54, 0.15, 0.35, 12

        if gacc_region == 'SWCC' or gacc_region == 'swcc' or gacc_region == 'SW' or gacc_region == 'sw':

            x1, x2, y, x_size, fontsize = 0.13, 0.54, 0.18, 0.35, 12

        if gacc_region == 'SACC' or gacc_region == 'sacc' or gacc_region == 'SE' or gacc_region == 'se':

            x1, x2, y, x_size, fontsize = 0.13, 0.54, 0.18, 0.35, 12

        if gacc_region == 'EACC' or gacc_region == 'eacc' or gacc_region == 'E' or gacc_region == 'e':

            x1, x2, y, x_size, fontsize = 0.13, 0.54, 0.16, 0.35, 12

        if gacc_region == 'PNW' or gacc_region == 'pnw' or gacc_region == 'NWCC' or gacc_region == 'nwcc' or gacc_region == 'NW' or gacc_region == 'nw':

            x1, x2, y, x_size, fontsize = 0.17, 0.54, 0.07, 0.3, 12

    return x1, x2, y, x_size, fontsize
        



import firewxpy.RTMA as rtma

import firewxpy.SPC_Outlook_Graphics as spc

from firewxpy.standard import plot_creation_time

from firewxpy.data_access import RTMA, NDFD_GRIDS, obs, model_data, FEMS

from firewxpy.NWS_Forecasts import temperature as nws_temperature_forecast
from firewxpy.NWS_Forecasts import relative_humidity as nws_relative_humidity_forecast
from firewxpy.NWS_Forecasts import critical_firewx as nws_critical_firewx_forecast

from firewxpy.forecast_models import dynamics as model_dynamics
from firewxpy.forecast_models import temperature as model_temperature
from firewxpy.forecast_models import relative_humidity as model_relative_humidity
from firewxpy.forecast_models import critical_firewx_conditions as model_critical_firewx_conditions
from firewxpy.forecast_models import precipitation as model_precipitation

from firewxpy.cross_sections import time_cross_sections, two_point_cross_sections

from firewxpy.observations import gridded_observations as gridded_obs
from firewxpy.observations import METAR_Observations as metar_obs
from firewxpy.observations import scatter_observations as scatter_obs

from firewxpy.soundings import plot_observed_sounding, plot_observed_sounding_custom_date_time, plot_forecast_soundings

from firewxpy.dims import get_metar_mask

from firewxpy.sawti import sawti

from firewxpy.solar_information import plot_daily_solar_information



# THIS FILE CONTAINS FUNCTIONS THAT RETURN VARIOUS TYPES OF GEOMETRIES FOR PLOTTING
# PYTHON DEPENDENCIES:
# 1) MATPLOTLIB
# 2) CARTOPY
#
#  (C) METEOROLOGIST ERIC J. DREWITZ
#               USDA/USFS

###### IMPORTS ################
import urllib.request
import os
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import warnings
warnings.filterwarnings('ignore')

from cartopy.io.shapereader import Reader
from cartopy.feature import ShapelyFeature
from firewxpy.utilities import file_functions

def get_shapes(file_path):

    try:
        shape_feature = ShapelyFeature(Reader(file_path).geometries(),
                                       ccrs.PlateCarree(), facecolor=(0,0,0,0), edgecolor='black')

        fname = os.path.basename(file_path)
    
        return shape_feature

    except Exception as a:
        error = shape_file_error()
        print(error)

    
def shape_file_error():
    error_msg = f"""
    
    WARNING: COULD NOT FIND FILE. PLEASE MAKE SURE YOU HAVE THE CORRECT FILE PATH. 

    ALSO, BE SURE TO HAVE ALL THE FILES AS FOLLOWS LOCATED IN THE FOLDER.
    HERE IS AN EXAMPLE WITH USING THE PSA BOUNDARY SHAPE FILES. 

    National_PSA_Current.shp
    National_PSA_Current.shx
    National_PSA_Current.xml
    National_PSA_Current.cpg
    National_PSA_Current.dbf
    National_PSA_Current.prj

    THE .SHP FILE IS THE FILE THAT NEEDS TO BE USED IN THE FUNCTION. 

    """
    return error_msg


def download_shape_files():

    # Defines file paths for different shapefiles
    cwa_path = f"NWS CWA Boundaries"
    fwz_path = f"NWS Fire Weather Zones"
    pz_path = f"NWS Public Zones"
    psa_path = f"PSA Shapefiles"
    gacc_path = f"GACC Boundaries Shapefiles"

    # CWAs
    
    if os.path.exists(cwa_path):
        pass
    else:
        # Makes new folder
        print("Welcome First Time FireWxPy User!\nLet me set you up automatically by downloading & installing all the shapefiles while also building the Weather Data directory for you.\nMy Motto is: 'Let the FireWxPy functions do the work so you don't have to!'\nSetting Up...")
        os.mkdir("NWS CWA Boundaries")
        # Downloads the CWA Shapefiles
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_CWA_Boundaries/w_05mr24.dbf', 'w_05mr24.dbf')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_CWA_Boundaries/w_05mr24.prj', 'w_05mr24.prj')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_CWA_Boundaries/w_05mr24.shx', 'w_05mr24.shx')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_CWA_Boundaries/w_05mr24.zip', 'w_05mr24.zip')
        # Moves files to new folder
        os.replace('w_05mr24.zip', f"NWS CWA Boundaries/w_05mr24.zip")
        os.replace('w_05mr24.dbf', f"NWS CWA Boundaries/w_05mr24.dbf")
        os.replace('w_05mr24.prj', f"NWS CWA Boundaries/w_05mr24.prj")
        os.replace('w_05mr24.shx', f"NWS CWA Boundaries/w_05mr24.shx")

    # Fire Weather Zones

    if os.path.exists(fwz_path):
        pass
    else:
        # Makes new folder
        os.mkdir("NWS Fire Weather Zones")
        # Downloads the FWZ Shapefiles
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_Fire_Weather_Zones/fz05mr24.zip', 'fz05mr24.zip')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_Fire_Weather_Zones/fz05mr24.dbf', 'fz05mr24.dbf')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_Fire_Weather_Zones/fz05mr24.prj', 'fz05mr24.prj')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_Fire_Weather_Zones/fz05mr24.shx', 'fz05mr24.shx')
        # Moves files to new folder
        os.replace('fz05mr24.zip', f"NWS Fire Weather Zones/fz05mr24.zip")
        os.replace('fz05mr24.dbf', f"NWS Fire Weather Zones/fz05mr24.dbf")
        os.replace('fz05mr24.prj', f"NWS Fire Weather Zones/fz05mr24.prj")
        os.replace('fz05mr24.shx', f"NWS Fire Weather Zones/fz05mr24.shx")

    # Public Zones

    if os.path.exists(pz_path):
        pass
    else:
        # Makes new folder
        os.mkdir("NWS Public Zones")
        # Downloads files
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_Public_Zones/z_05mr24.zip', 'z_05mr24.zip')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_Public_Zones/z_05mr24.dbf', 'z_05mr24.dbf')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_Public_Zones/z_05mr24.prj', 'z_05mr24.prj')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/NWS_Public_Zones/z_05mr24.shx', 'z_05mr24.shx')
        # Moves files to new folder
        os.replace('z_05mr24.zip', f"NWS Public Zones/z_05mr24.zip")
        os.replace('z_05mr24.dbf', f"NWS Public Zones/z_05mr24.dbf")
        os.replace('z_05mr24.prj', f"NWS Public Zones/z_05mr24.prj")
        os.replace('z_05mr24.shx', f"NWS Public Zones/z_05mr24.shx")

    # GACC Boundaries
    if os.path.exists(gacc_path):
        pass
    else:
        # Makes new folder
        os.mkdir("GACC Boundaries Shapefiles")
        # Downloads files
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/GACC%20Boundaries%20Shapefiles/National_GACC_Boundaries.xml', 'National_GACC_Boundaries.xml')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/GACC%20Boundaries%20Shapefiles/National_GACC_Current.cpg', 'National_GACC_Current.cpg')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/GACC%20Boundaries%20Shapefiles/National_GACC_Current.dbf', 'National_GACC_Current.dbf')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/GACC%20Boundaries%20Shapefiles/National_GACC_Current.prj', 'National_GACC_Current.prj')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/GACC%20Boundaries%20Shapefiles/National_GACC_Current.shp', 'National_GACC_Current.shp')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/GACC%20Boundaries%20Shapefiles/National_GACC_Current.shx', 'National_GACC_Current.shx')
        # Moves files to folder
        os.replace('National_GACC_Boundaries.xml', f"GACC Boundaries Shapefiles/National_GACC_Boundaries.xml")
        os.replace('National_GACC_Current.cpg', f"GACC Boundaries Shapefiles/National_GACC_Current.cpg")
        os.replace('National_GACC_Current.dbf', f"GACC Boundaries Shapefiles/National_GACC_Current.dbf")
        os.replace('National_GACC_Current.prj', f"GACC Boundaries Shapefiles/National_GACC_Current.prj")
        os.replace('National_GACC_Current.shp', f"GACC Boundaries Shapefiles/National_GACC_Current.shp")
        os.replace('National_GACC_Current.shx', f"GACC Boundaries Shapefiles/National_GACC_Current.shx")

    # PSA Boundaries
    if os.path.exists(psa_path):
        pass
    else:
        # Makes new folder
        os.mkdir("PSA Shapefiles")
        # Downloads files

        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/PSA%20Shapefiles/National_PSA_Current.xml', 'National_PSA_Current.xml')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/PSA%20Shapefiles/National_PSA_Current.cpg', 'National_PSA_Current.cpg')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/PSA%20Shapefiles/National_PSA_Current.dbf', 'National_PSA_Current.dbf')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/PSA%20Shapefiles/National_PSA_Current.prj', 'National_PSA_Current.prj')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/PSA%20Shapefiles/National_PSA_Current.shp', 'National_PSA_Current.shp')
        urllib.request.urlretrieve('https://raw.githubusercontent.com/edrewitz/FireWxPy/main/shapefiles/PSA%20Shapefiles/National_PSA_Current.shx', 'National_PSA_Current.shx')

        # Moves files to folder
        os.replace('National_PSA_Current.xml', f"PSA Shapefiles/National_PSA_Current.xml")
        os.replace('National_PSA_Current.cpg', f"PSA Shapefiles/National_PSA_Current.cpg")
        os.replace('National_PSA_Current.dbf', f"PSA Shapefiles/National_PSA_Current.dbf")
        os.replace('National_PSA_Current.prj', f"PSA Shapefiles/National_PSA_Current.prj")
        os.replace('National_PSA_Current.shp', f"PSA Shapefiles/National_PSA_Current.shp")
        os.replace('National_PSA_Current.shx', f"PSA Shapefiles/National_PSA_Current.shx")
        print("Set Up Complete! Have Fun!")



def import_shapefiles(file_path, line_color, boundary_type):

    r'''
    This function reads and returns the shapefiles (.shp) files from a specific file location. 

    Required Arguments: 1) file_path (String) - The file location of the SHP files. 
                        2) line_color (String) - The color the user wishes to display for the borders in the shapefile. 
                        3) boundary_type (String) - The type of geographical boundaries the user wishes to use. 
                                                    This is necessary because the NWS boundaries have a large file size which to be able to host on github, the files need to be zipped so we need to unzip and extract those files. 

    Returns: 1) The shapefile borders the user wishes to import into the weather graphics. 

    '''
    file_path = file_path 
    line_color = line_color
    boundary_type = boundary_type

    download_shape_files()

    if boundary_type == 'cwa':

        if os.path.exists(f"NWS CWA Boundaries/w_05mr24.shp"):
            pass
        else:
            file_functions.extract_zipped_files(f"NWS CWA Boundaries/w_05mr24.zip", f"NWS CWA Boundaries")

    elif boundary_type == 'fwz':

        if os.path.exists(f"NWS Fire Weather Zones/fz05mr24.shp"):
            pass
        else:
            file_functions.extract_zipped_files(f"NWS Fire Weather Zones/fz05mr24.zip", f"NWS Fire Weather Zones")

    elif boundary_type == 'pz':

        if os.path.exists(f"NWS Public Zones/z_05mr24.shp"):
            pass
        else:
            file_functions.extract_zipped_files(f"NWS Public Zones/z_05mr24.zip", f"NWS Public Zones")

    else:
        pass

PSAs = import_shapefiles(f"PSA Shapefiles/National_PSA_Current.shp", 'black', 'psa')

GACC = import_shapefiles(f"GACC Boundaries Shapefiles/National_GACC_Current.shp", 'black', 'gacc')

CWAs = import_shapefiles(f"NWS CWA Boundaries/w_05mr24.shp", 'black', 'cwa')

FWZs = import_shapefiles(f"NWS Fire Weather Zones/fz05mr24.shp", 'black', 'fwz')

PZs = import_shapefiles(f"NWS Public Zones/z_05mr24.shp", 'black', 'pz')




"""
This file hosts the functions to plot the latest SPC Outlooks. 


This file was written by Meteorologist Eric J. Drewitz 

            (C) Meteorologist Eric J. Drewitz 
                        USDA/USFS

"""
#### IMPORTS ####

import matplotlib as mpl
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import cartopy.feature as cfeature
import metpy.plots as mpplots
import metpy.calc as mpcalc
import numpy as np
import firewxpy.geometry as geometry
import firewxpy.colormaps as colormaps
import firewxpy.settings as settings
import firewxpy.standard as standard
import firewxpy.dims as dims
import os
import pandas as pd
import xarray as xr
import warnings
warnings.filterwarnings('ignore')

from metpy.plots import USCOUNTIES
from matplotlib.patheffects import withStroke
from firewxpy.calc import scaling, unit_conversion
from firewxpy.utilities import file_functions
from firewxpy.data_access import NDFD_GRIDS
from firewxpy.parsers import NDFD
from metpy.units import units
from dateutil import tz
from datetime import datetime, timedelta

mpl.rcParams['font.weight'] = 'bold'
props = dict(boxstyle='round', facecolor='wheat', alpha=1)

mpl.rcParams['ytick.labelsize'] = 6

provinces = cfeature.NaturalEarthFeature(category='cultural', 
    name='admin_1_states_provinces_lines', scale='50m', facecolor='none', edgecolor='k')

mapcrs = ccrs.PlateCarree()
datacrs = ccrs.PlateCarree()

local_time, utc_time = standard.plot_creation_time()
timezone = standard.get_timezone_abbreviation()
tzone = standard.get_timezone()

def plot_critical_fire_weather_risk_outlook(western_bound=None, eastern_bound=None, southern_bound=None, northern_bound=None, shrink=0.7, show_rivers=True, reference_system='States & Counties', show_state_borders=False, show_county_borders=False, show_gacc_borders=False, show_psa_borders=False, show_cwa_borders=False, show_nws_firewx_zones=False, show_nws_public_zones=False, state_border_linewidth=1, county_border_linewidth=0.25, gacc_border_linewidth=1, psa_border_linewidth=1, cwa_border_linewidth=1, nws_firewx_zones_linewidth=0.5, nws_public_zones_linewidth=0.5, state_border_linestyle='-', county_border_linestyle='-', gacc_border_linestyle='-', psa_border_linestyle='-', cwa_border_linestyle='-', nws_firewx_zones_linestyle='-', nws_public_zones_linestyle='-', data=False, ds_short=None, ds_extended=None, state='conus', gacc_region=None, x1=0.01, y1=-0.03, x2=0.725, y2=-0.025, x3=0.01, y3=0.01, signature_fontsize=6, stamp_fontsize=5):


    r'''
    This function plots the latest available NOAA/NWS SPC Critical Fire Weather Risk Outlook. 

    Required Arguments: None

    Optional Arguments: 

    1) western_bound (Integer or Float) - Default = None. Western extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Negative values denote the western hemisphere and positive 
       values denote the eastern hemisphere. 

    2) eastern_bound (Integer or Float) - Default = None. Eastern extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Negative values denote the western hemisphere and positive 
       values denote the eastern hemisphere. 

    3) southern_bound (Integer or Float) - Default = None. Southern extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Positive values denote the northern hemisphere and negative 
       values denote the southern hemisphere. 

    4) northern_bound (Integer or Float) - Default = None. Northern extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Positive values denote the northern hemisphere and negative 
       values denote the southern hemisphere.
    
    5) shrink (Integer or Float) - Default = 0.7. This is how the colorbar is sized to the figure. 
       This is a feature of matplotlib, as per their definition, the shrink is:
       "Fraction by which to multiply the size of the colorbar." 
       This should only be changed if the user wishes to change the size of the colorbar. 
       Preset values are called from the settings module for each state and/or gacc_region.

    6) show_rivers (Boolean) - If set to True, rivers will display. If set to False, county borders will not display. 
        Default setting is True. Users should change this value to False if they wish to hide rivers.

    7) reference_system (String) - Default = 'States & Counties'. The georgraphical reference system with respect to the borders on the map. If the user
        wishes to use a reference system not on this list, please see items 17-23. 
        Reference Systems: 
        
        1) 'States & Counties'
        2) 'States Only'
        3) 'GACC Only'
        4) 'GACC & PSA'
        5) 'CWA Only'
        6) 'NWS CWAs & NWS Public Zones'
        7) 'NWS CWAs & NWS Fire Weather Zones'
        8) 'NWS CWAs & Counties'
        9) 'GACC & PSA & NWS Fire Weather Zones'
        10) 'GACC & PSA & NWS Public Zones'
        11) 'GACC & PSA & NWS CWA'
        12) 'GACC & PSA & Counties'
        13) 'GACC & Counties'
                           

    8) show_state_borders (Boolean) - If set to True, state borders will display. If set to False, state borders will not display. 
        Default setting is False. Users should change this value to False if they wish to hide state borders. 

    9) show_county_borders (Boolean) - If set to True, county borders will display. If set to False, county borders will not display. 
        Default setting is False. Users should change this value to False if they wish to hide county borders. 

    10) show_gacc_borders (Boolean) - If set to True, GACC (Geographic Area Coordination Center) borders will display. If set to False, GACC borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display GACC borders. 

    11) show_psa_borders (Boolean) - If set to True, PSA (Predictive Services Area) borders will display. If set to False, PSA borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display PSA borders.

    12) show_cwa_borders (Boolean) - If set to True, CWA borders will display. If set to False, CWA borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display CWA borders.

    13) show_nws_firewx_zones (Boolean) - If set to True, NWS FWZ borders will display. If set to False, NWS FWZ borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display NWS FWZ borders.

    14) show_nws_public_zones (Boolean) - If set to True, NWS Public Zone borders will display. If set to False, NWS Public Zone borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display NWS Public Zone borders.

    15) state_border_linewidth (Integer) - Linewidth (thickness) of the state borders. Default setting is 2. 

    16) county_border_linewidth (Integer) - Linewidth (thickness) of the county borders. Default setting is 1. 

    17) gacc_border_linewidth (Integer) - Linewidth (thickness) of the GACC borders. Default setting is 2. 

    18) psa_border_linewidth (Integer) - Linewidth (thickness) of the PSA borders. Default setting is 1. 

    19) state_border_linestyle (String) - Linestyle of the state borders. Default is a solid line. 
        To change to a dashed line, users should set state_border_linestyle='--'. 

    20) county_border_linestyle (String) - Linestyle of the county borders. Default is a solid line. 
        To change to a dashed line, users should set county_border_linestyle='--'. 

    21) gacc_border_linestyle (String) - Linestyle of the GACC borders. Default is a solid line. 
        To change to a dashed line, users should set gacc_border_linestyle='--'. 

    22) psa_border_linestyle (String) - Linestyle of the PSA borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    23) cwa_border_linestyle (String) - Linestyle of the CWA borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    24) nws_firewx_zones_linestyle (String) - Linestyle of the NWS FWZ borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    25) nws_public_zones_linestyle (String) - Linestyle of the NWS Public Zone borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    26) data (Boolean) - Default = False. This tells the function if the user is downloading the data outside of the function
        and passing the data in or if the function needs to download the data. A value of False means the data
        is downloaded inside of the function while a value of True means the user is downloading the data outside
        of the function and passing it in. For users who intend to make a lot of graphics for a lot of different 
        things, it is recommended to set this value to True and download the data outside of the function and pass
        it in so that the amount of data requests on the host servers can be minimized. 

    27) ds_short (xarray.dataarray) - The xarray data array of the downloaded NWS Short-Term Forecast dataset from the NWS FTP Server. 
        This setting is only to be changed if the user wants to limit the amount of downloads from the 
        NWS FTP Server. Default setting is None. Please see the documentation for the data_access module 
        if the user wishes to download the data outside of this function. 

    28) ds_extended (xarray.dataarray) - The xarray data array of the downloaded NWS Extended Forecast dataset from the NWS FTP Server. 
        This setting is only to be changed if the user wants to limit the amount of downloads from the 
        NWS FTP Server. Default setting is None. Please see the documentation for the data_access module 
        if the user wishes to download the data outside of this function. 

    29) state (String) - The two letter state abbreviation for the state the user wishes to make the graphic for. 
        If the user wishes to make a graphic for the entire CONUS, there are 2 acceptable abbreviations: 'CONUS' or 'conus'. 
        Example: If the user wishes to make a plot for the state of California both 'CA' or 'ca' are
        acceptable. Default setting is 'conus'. If the user wishes to make a plot based on gacc_region, this value must be 
        changed to None. 

    30) gacc_region (String) - The abbreviation for each of the 10 GACC regions. Default setting is None. 
        If the user wishes to make a plot based on GACC Region than state, the state variable must be set to 
        None and the gacc_region variable must be set to one of the acceptable abbreviations. 

        Here is a list of acceptable gacc_region abbreviations:

        South Ops: 'OSCC' or 'oscc' or 'SOPS' or 'sops'
        
        North Ops: 'ONCC' or 'oncc' or 'NOPS' or 'nops'
        
        Great Basin: 'GBCC' or 'gbcc' or 'GB' or 'gb'
        
        Northern Rockies: 'NRCC' or 'nrcc' or 'NR' or 'nr'
        
        Rocky Mountain: 'RMCC' or 'rmcc' or 'RM' or 'rm'
        
        Southwest: 'SWCC' or 'swcc' or 'SW' or 'sw'
        
        Southern: 'SACC' or 'sacc' or 'SE' or 'se'
        
        Eastern: 'EACC' or 'eacc' or 'E' or 'e'
        
        Pacific Northwest: 'PNW' or 'pnw' or 'NWCC' or 'nwcc' or 'NW' or 'nw'
        
        Alaska: Setting state='AK' or state='ak' suffices here. Leave gacc_region=None and set the state variable as shown. 

    31) x1 (Float) - Default = 0.01. The x-position of the signature text box with respect to the axis of the image. 

    32) y1 (Float) - Default = -0.03. The y-position of the signature text box with respect to the axis of the image. 

    33) x2 (Float) - Default = 0.725. The x-position of the timestamp text box with respect to the axis of the image.

    34) y2 (Float) - Default = -0.025. The y-position of the timestamp text box with respect to the axis of the image.

    35) x3 (Float) - Default = 0.01. The x-position of the reference system text box with respect to the axis of the image.

    36) y3 (Float) - Default = 0.01. The y-position of the reference system text box with respect to the axis of the image.     

    37) signature_fontsize (Integer) - Default = 6. The fontsize of the signature. This is only to be changed when making a custom plot. 

    38) stamp_fontsize (Integer) - Default = 5. The fontsize of the timestamp and reference system text. This is only to be changed when making a custom plot. 

    Return: Saves individual images to f:Weather Data/SPC Outlooks/Critical Fire Weather Risk Outlook/{reference_system}. 
    '''
    PSAs = geometry.get_shapes(f"PSA Shapefiles/National_PSA_Current.shp")
    
    GACC = geometry.get_shapes(f"GACC Boundaries Shapefiles/National_GACC_Current.shp")
    
    CWAs = geometry.get_shapes(f"NWS CWA Boundaries/w_05mr24.shp")
    
    FWZs = geometry.get_shapes(f"NWS Fire Weather Zones/fz05mr24.shp")
    
    PZs = geometry.get_shapes(f"NWS Public Zones/z_05mr24.shp")

    state = state

    if gacc_region != None:
        state = gacc_region
    else:
        state = state

    levels = np.arange(4, 12, 2)  

    cmap = colormaps.SPC_Critical_Fire_Weather_Risk_Outlook_colormap()

    reference_system = reference_system


    if reference_system == 'Custom' or reference_system == 'custom':
        show_state_borders = show_state_borders
        show_county_borders = show_county_borders
        show_gacc_borders = show_gacc_borders
        show_psa_borders = show_psa_borders
        show_cwa_borders = show_cwa_borders
        show_nws_firewx_zones = show_nws_firewx_zones
        show_nws_public_zones = show_nws_public_zones

    if reference_system != 'Custom' and reference_system != 'custom':
        
        show_state_borders = False
        show_county_borders = False
        show_gacc_borders = False
        show_psa_borders = False
        show_cwa_borders = False
        show_nws_firewx_zones = False
        show_nws_public_zones = False

        if reference_system == 'States Only':
            show_state_borders = True
        if reference_system == 'States & Counties':
            show_state_borders = True
            show_county_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                county_border_linewidth=0.25
        if reference_system == 'GACC Only':
            show_gacc_borders = True
        if reference_system == 'GACC & PSA':
            show_gacc_borders = True
            show_psa_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.25
        if reference_system == 'CWA Only':
            show_cwa_borders = True
        if reference_system == 'NWS CWAs & NWS Public Zones':
            show_cwa_borders = True
            show_nws_public_zones = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                nws_public_zones_linewidth=0.25
        if reference_system == 'NWS CWAs & NWS Fire Weather Zones':
            show_cwa_borders = True
            show_nws_firewx_zones = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                nws_firewx_zones_linewidth=0.25
        if reference_system == 'NWS CWAs & Counties':
            show_cwa_borders = True
            show_county_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                county_border_linewidth=0.25
        if reference_system == 'GACC & PSA & NWS Fire Weather Zones':
            show_gacc_borders = True
            show_psa_borders = True
            show_nws_firewx_zones = True
            nws_firewx_zones_linewidth=0.25
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.5
        if reference_system == 'GACC & PSA & NWS Public Zones':
            show_gacc_borders = True
            show_psa_borders = True
            show_nws_public_zones = True
            nws_public_zones_linewidth=0.25
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.5
        if reference_system == 'GACC & PSA & NWS CWA':
            show_gacc_borders = True
            show_psa_borders = True
            show_cwa_borders = True
            cwa_border_linewidth=0.25
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.5
        if reference_system == 'GACC & PSA & Counties':
            show_gacc_borders = True
            show_psa_borders = True
            show_county_borders = True
            county_border_linewidth=0.25
        if reference_system == 'GACC & Counties':
            show_gacc_borders = True
            show_county_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                county_border_linewidth=0.25
                
    try:  
        western_bound, eastern_bound, southern_bound, northern_bound, x1, y1, x2, y2, x3, y3, shrink, de, signature_fontsize, stamp_fontsize = settings.get_region_info('NAM', state)
        if state == 'AK' or state == 'ak':
           print(f"This product is only availiable for CONUS.\nSorry OCONUS folks...")
        if state != 'AK' and state != 'ak' or gacc_region != None:
            directory_name = '/SL.us008001/ST.opnl/DF.gr2/DC.ndfd/AR.conus/'

        sp, x, y = settings.get_sp_dims_and_textbox_coords(state)

    except Exception as e:
        western_bound = western_bound
        eastern_bound = eastern_bound
        southern_bound = southern_bound
        northern_bound = northern_bound
        x1=x1
        y1=y1
        x2=x2
        y2=y2
        x3=x3
        y3=y3
        directory_name = '/SL.us008001/ST.opnl/DF.gr2/DC.ndfd/AR.conus/'

    path, path_print = file_functions.noaa_graphics_paths(state, gacc_region, 'Critical Fire Weather Risk Outlook', reference_system, None, spc=True)

    for file in os.listdir(f"{path}"):
        try:
            os.remove(f"{path}/{file}")
        except Exception as e:
            pass
        
    if data == False:

        ds_short, ds_extended = NDFD_GRIDS.get_ndfd_grids_xarray(directory_name, 'ds.critfireo.bin', state)

    else:
        ds_short = ds_short
        ds_extended = ds_extended

    short_stop = len(ds_short['step'])
    extended_stop = len(ds_extended['step'])
    extended_start = short_stop + 1

    short_times = ds_short['valid_time']
    extended_times = ds_extended['valid_time']
    short_times = short_times.to_pandas()
    extended_times = extended_times.to_pandas()

    short_start_times, short_end_times, short_start_times_utc = NDFD.get_valid_times_xarray(ds_short, 24)
    extended_start_times, extended_end_times, extended_start_times_utc = NDFD.get_valid_times_xarray(ds_extended, 24)

    init_hr = 12
    hour = short_start_times_utc[0].hour
    dt = False
    if hour == init_hr:
        start = 0
        skip = False
    else:
        if local_time.hour <= 14:
            start = 0
            skip = False
            dt = True
            hours = hour - init_hr
        else:
            start = 1
            skip = True

    if skip == True:
        extended_start = extended_start - 1
    else:
        extended_start = extended_start

    for i in range(start, short_stop, 1):

        fname = f"A_Short_Term_{i}.png"
        if skip == False:
            index = 1 + i
        else:
            index = i

        fig = plt.figure(figsize=(12,12))
        fig.set_facecolor('aliceblue')
        ax = fig.add_subplot(1, 1, 1, projection=mapcrs)
        ax.set_extent([western_bound, eastern_bound, southern_bound, northern_bound], datacrs)
        ax.add_feature(cfeature.COASTLINE.with_scale('50m'), linewidth=0.75, zorder=9)
        ax.add_feature(cfeature.LAND, color='beige', zorder=1)
        ax.add_feature(cfeature.OCEAN, color='lightcyan', zorder=1)
        ax.add_feature(cfeature.LAKES, color='lightcyan', zorder=1)
        ax.add_feature(provinces, linewidth=1, zorder=1)
        
        if show_rivers == True:
            ax.add_feature(cfeature.RIVERS, color='lightcyan', zorder=4)
        else:
            pass
    
        if show_gacc_borders == True:
            ax.add_feature(GACC, linewidth=gacc_border_linewidth, linestyle=gacc_border_linestyle, zorder=6)
        else:
            pass
        if show_psa_borders == True:
            ax.add_feature(PSAs, linewidth=psa_border_linewidth, linestyle=psa_border_linestyle, zorder=5)
        else:
            pass
        if show_county_borders == True:
            ax.add_feature(USCOUNTIES, linewidth=county_border_linewidth, linestyle=county_border_linestyle, zorder=5)
        else:
            pass
        if show_state_borders == True:
            ax.add_feature(cfeature.STATES, linewidth=state_border_linewidth, linestyle=state_border_linestyle, edgecolor='black', zorder=6)
        else:
            pass
        if show_cwa_borders == True:
            ax.add_feature(CWAs, linewidth=cwa_border_linewidth, linestyle=cwa_border_linestyle, zorder=5)
        else:
            pass
        if show_nws_firewx_zones == True:
            ax.add_feature(FWZs, linewidth=nws_firewx_zones_linewidth, linestyle=nws_firewx_zones_linestyle, zorder=5)
        else:
            pass
        if show_nws_public_zones == True:
            ax.add_feature(PZs, linewidth=nws_public_zones_linewidth, linestyle=nws_public_zones_linestyle, zorder=5)
        else:
            pass  

        start = short_start_times[i]
        end = short_end_times[i]             

        cs = ax.contourf(ds_short['longitude'][:, :], ds_short['latitude'][:, :], ds_short['unknown'][i, :, :], levels=levels, cmap=cmap, transform=datacrs, alpha=0.5)
        cbar = fig.colorbar(cs, shrink=shrink, pad=0.01, location='right', ticks=[5, 7, 9])
        cbar.ax.set_yticklabels(['Elevated', 'Critical', 'Extreme'])

        plt.title(f"SPC Outlook [Day {index}]\nCritical Fire Weather Outlook", fontsize=8, fontweight='bold', loc='left')
        if dt == False:
            plt.title(f"Valid: {start.strftime(f'%a %b %d')}", fontsize=7, fontweight='bold', loc='right')
        else:
            start_hour = short_start_times[0].hour - hours
            end_hour = short_end_times[0].hour - hours
            plt.title(f"Valid: {start.strftime(f'%a %b %d')}", fontsize=7, fontweight='bold', loc='right')

        ax.text(x1, y1, "Plot Created With FireWxPy (C) Eric J. Drewitz " +utc_time.strftime('%Y')+" | Data Source: NOAA/NCEP/SPC", transform=ax.transAxes, fontsize=signature_fontsize, fontweight='bold', bbox=props)
        ax.text(x2, y2, "Image Created: " + local_time.strftime(f'%m/%d/%Y %H:%M {timezone}') + " (" + utc_time.strftime('%H:%M UTC') + ")", transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props)
        ax.text(x3, y3, "Reference System: "+reference_system, transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props, zorder=11)  

        fig.savefig(f"{path}/{fname}", bbox_inches='tight')
    print(f"Saved short-term forecast graphics to {path_print}")


    for i in range(0, extended_stop, 1):

        fname = f"B_Extended_Term_{i}.png"

        index = extended_start + i

        fig = plt.figure(figsize=(12,12))
        fig.set_facecolor('aliceblue')
        ax = fig.add_subplot(1, 1, 1, projection=mapcrs)
        ax.set_extent([western_bound, eastern_bound, southern_bound, northern_bound], datacrs)
        ax.add_feature(cfeature.COASTLINE.with_scale('50m'), linewidth=0.75, zorder=9)
        ax.add_feature(cfeature.LAND, color='beige', zorder=1)
        ax.add_feature(cfeature.OCEAN, color='lightcyan', zorder=1)
        ax.add_feature(cfeature.LAKES, color='lightcyan', zorder=1)
        ax.add_feature(provinces, linewidth=1, zorder=1)
        if show_rivers == True:
            ax.add_feature(cfeature.RIVERS, color='lightcyan', zorder=4)
        else:
            pass
    
        if show_gacc_borders == True:
            ax.add_feature(GACC, linewidth=gacc_border_linewidth, linestyle=gacc_border_linestyle, zorder=6)
        else:
            pass
        if show_psa_borders == True:
            ax.add_feature(PSAs, linewidth=psa_border_linewidth, linestyle=psa_border_linestyle, zorder=5)
        else:
            pass
        if show_county_borders == True:
            ax.add_feature(USCOUNTIES, linewidth=county_border_linewidth, linestyle=county_border_linestyle, zorder=5)
        else:
            pass
        if show_state_borders == True:
            ax.add_feature(cfeature.STATES, linewidth=state_border_linewidth, linestyle=state_border_linestyle, edgecolor='black', zorder=6)
        else:
            pass
        if show_cwa_borders == True:
            ax.add_feature(CWAs, linewidth=cwa_border_linewidth, linestyle=cwa_border_linestyle, zorder=5)
        else:
            pass
        if show_nws_firewx_zones == True:
            ax.add_feature(FWZs, linewidth=nws_firewx_zones_linewidth, linestyle=nws_firewx_zones_linestyle, zorder=5)
        else:
            pass
        if show_nws_public_zones == True:
            ax.add_feature(PZs, linewidth=nws_public_zones_linewidth, linestyle=nws_public_zones_linestyle, zorder=5)
        else:
            pass  

        start = extended_start_times[i]
        end = extended_end_times[i]

        cs = ax.contourf(ds_extended['longitude'][:, :], ds_extended['latitude'][:, :], ds_extended['unknown'][i, :, :], levels=levels, cmap=cmap, transform=datacrs, alpha=0.5)

        cbar = fig.colorbar(cs, shrink=shrink, pad=0.01, location='right', ticks=[5, 7, 9])
        cbar.ax.set_yticklabels(['Elevated', 'Critical', 'Extreme'])

        plt.title(f"SPC Outlook [Day {index}]\nCritical Fire Weather Outlook", fontsize=8, fontweight='bold', loc='left')
        plt.title(f"Valid: {start.strftime(f'%a %b %d')}", fontsize=7, fontweight='bold', loc='right')

        ax.text(x1, y1, "Plot Created With FireWxPy (C) Eric J. Drewitz " +utc_time.strftime('%Y')+" | Data Source: NOAA/NCEP/SPC", transform=ax.transAxes, fontsize=signature_fontsize, fontweight='bold', bbox=props)
        ax.text(x2, y2, "Image Created: " + local_time.strftime(f'%m/%d/%Y %H:%M {timezone}') + " (" + utc_time.strftime('%H:%M UTC') + ")", transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props)
        ax.text(x3, y3, "Reference System: "+reference_system, transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props, zorder=11)   

        fig.savefig(f"{path}/{fname}", bbox_inches='tight')
    print(f"Saved extended forecast graphics to {path_print}")


def plot_dry_lightning_outlook(western_bound=None, eastern_bound=None, southern_bound=None, northern_bound=None, shrink=0.7, show_rivers=True, reference_system='States & Counties', show_state_borders=False, show_county_borders=False, show_gacc_borders=False, show_psa_borders=False, show_cwa_borders=False, show_nws_firewx_zones=False, show_nws_public_zones=False, state_border_linewidth=1, county_border_linewidth=0.25, gacc_border_linewidth=1, psa_border_linewidth=1, cwa_border_linewidth=1, nws_firewx_zones_linewidth=0.5, nws_public_zones_linewidth=0.5, state_border_linestyle='-', county_border_linestyle='-', gacc_border_linestyle='-', psa_border_linestyle='-', cwa_border_linestyle='-', nws_firewx_zones_linestyle='-', nws_public_zones_linestyle='-', data=False, ds_short=None, ds_extended=None, state='conus', gacc_region=None, x1=0.01, y1=-0.03, x2=0.725, y2=-0.025, x3=0.01, y3=0.01, signature_fontsize=6, stamp_fontsize=5):


    r'''
    This function plots the latest available NOAA/NWS SPC Dry Lightning Outlook. 

    Required Arguments: None

    Optional Arguments: 

    1) western_bound (Integer or Float) - Default = None. Western extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Negative values denote the western hemisphere and positive 
       values denote the eastern hemisphere. 

    2) eastern_bound (Integer or Float) - Default = None. Eastern extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Negative values denote the western hemisphere and positive 
       values denote the eastern hemisphere. 

    3) southern_bound (Integer or Float) - Default = None. Southern extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Positive values denote the northern hemisphere and negative 
       values denote the southern hemisphere. 

    4) northern_bound (Integer or Float) - Default = None. Northern extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Positive values denote the northern hemisphere and negative 
       values denote the southern hemisphere.
    
    5) shrink (Integer or Float) - Default = 0.7. This is how the colorbar is sized to the figure. 
       This is a feature of matplotlib, as per their definition, the shrink is:
       "Fraction by which to multiply the size of the colorbar." 
       This should only be changed if the user wishes to change the size of the colorbar. 
       Preset values are called from the settings module for each state and/or gacc_region.

    6) show_rivers (Boolean) - If set to True, rivers will display. If set to False, county borders will not display. 
        Default setting is True. Users should change this value to False if they wish to hide rivers.

    7) reference_system (String) - Default = 'States & Counties'. The georgraphical reference system with respect to the borders on the map. If the user
        wishes to use a reference system not on this list, please see items 17-23. 
        Reference Systems: 
        
        1) 'States & Counties'
        2) 'States Only'
        3) 'GACC Only'
        4) 'GACC & PSA'
        5) 'CWA Only'
        6) 'NWS CWAs & NWS Public Zones'
        7) 'NWS CWAs & NWS Fire Weather Zones'
        8) 'NWS CWAs & Counties'
        9) 'GACC & PSA & NWS Fire Weather Zones'
        10) 'GACC & PSA & NWS Public Zones'
        11) 'GACC & PSA & NWS CWA'
        12) 'GACC & PSA & Counties'
        13) 'GACC & Counties'
                           

    8) show_state_borders (Boolean) - If set to True, state borders will display. If set to False, state borders will not display. 
        Default setting is False. Users should change this value to False if they wish to hide state borders. 

    9) show_county_borders (Boolean) - If set to True, county borders will display. If set to False, county borders will not display. 
        Default setting is False. Users should change this value to False if they wish to hide county borders. 

    10) show_gacc_borders (Boolean) - If set to True, GACC (Geographic Area Coordination Center) borders will display. If set to False, GACC borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display GACC borders. 

    11) show_psa_borders (Boolean) - If set to True, PSA (Predictive Services Area) borders will display. If set to False, PSA borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display PSA borders.

    12) show_cwa_borders (Boolean) - If set to True, CWA borders will display. If set to False, CWA borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display CWA borders.

    13) show_nws_firewx_zones (Boolean) - If set to True, NWS FWZ borders will display. If set to False, NWS FWZ borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display NWS FWZ borders.

    14) show_nws_public_zones (Boolean) - If set to True, NWS Public Zone borders will display. If set to False, NWS Public Zone borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display NWS Public Zone borders.

    15) state_border_linewidth (Integer) - Linewidth (thickness) of the state borders. Default setting is 2. 

    16) county_border_linewidth (Integer) - Linewidth (thickness) of the county borders. Default setting is 1. 

    17) gacc_border_linewidth (Integer) - Linewidth (thickness) of the GACC borders. Default setting is 2. 

    18) psa_border_linewidth (Integer) - Linewidth (thickness) of the PSA borders. Default setting is 1. 

    19) state_border_linestyle (String) - Linestyle of the state borders. Default is a solid line. 
        To change to a dashed line, users should set state_border_linestyle='--'. 

    20) county_border_linestyle (String) - Linestyle of the county borders. Default is a solid line. 
        To change to a dashed line, users should set county_border_linestyle='--'. 

    21) gacc_border_linestyle (String) - Linestyle of the GACC borders. Default is a solid line. 
        To change to a dashed line, users should set gacc_border_linestyle='--'. 

    22) psa_border_linestyle (String) - Linestyle of the PSA borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    23) cwa_border_linestyle (String) - Linestyle of the CWA borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    24) nws_firewx_zones_linestyle (String) - Linestyle of the NWS FWZ borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    25) nws_public_zones_linestyle (String) - Linestyle of the NWS Public Zone borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    26) data (Boolean) - Default = False. This tells the function if the user is downloading the data outside of the function
        and passing the data in or if the function needs to download the data. A value of False means the data
        is downloaded inside of the function while a value of True means the user is downloading the data outside
        of the function and passing it in. For users who intend to make a lot of graphics for a lot of different 
        things, it is recommended to set this value to True and download the data outside of the function and pass
        it in so that the amount of data requests on the host servers can be minimized. 

    27) ds_short (xarray.dataarray) - The xarray data array of the downloaded NWS Short-Term Forecast dataset from the NWS FTP Server. 
        This setting is only to be changed if the user wants to limit the amount of downloads from the 
        NWS FTP Server. Default setting is None. Please see the documentation for the data_access module 
        if the user wishes to download the data outside of this function. 

    28) ds_extended (xarray.dataarray) - The xarray data array of the downloaded NWS Extended Forecast dataset from the NWS FTP Server. 
        This setting is only to be changed if the user wants to limit the amount of downloads from the 
        NWS FTP Server. Default setting is None. Please see the documentation for the data_access module 
        if the user wishes to download the data outside of this function. 

    29) state (String) - The two letter state abbreviation for the state the user wishes to make the graphic for. 
        If the user wishes to make a graphic for the entire CONUS, there are 2 acceptable abbreviations: 'CONUS' or 'conus'. 
        Example: If the user wishes to make a plot for the state of California both 'CA' or 'ca' are
        acceptable. Default setting is 'conus'. If the user wishes to make a plot based on gacc_region, this value must be 
        changed to None. 

    30) gacc_region (String) - The abbreviation for each of the 10 GACC regions. Default setting is None. 
        If the user wishes to make a plot based on GACC Region than state, the state variable must be set to 
        None and the gacc_region variable must be set to one of the acceptable abbreviations. 

        Here is a list of acceptable gacc_region abbreviations:

        South Ops: 'OSCC' or 'oscc' or 'SOPS' or 'sops'
        
        North Ops: 'ONCC' or 'oncc' or 'NOPS' or 'nops'
        
        Great Basin: 'GBCC' or 'gbcc' or 'GB' or 'gb'
        
        Northern Rockies: 'NRCC' or 'nrcc' or 'NR' or 'nr'
        
        Rocky Mountain: 'RMCC' or 'rmcc' or 'RM' or 'rm'
        
        Southwest: 'SWCC' or 'swcc' or 'SW' or 'sw'
        
        Southern: 'SACC' or 'sacc' or 'SE' or 'se'
        
        Eastern: 'EACC' or 'eacc' or 'E' or 'e'
        
        Pacific Northwest: 'PNW' or 'pnw' or 'NWCC' or 'nwcc' or 'NW' or 'nw'
        
        Alaska: Setting state='AK' or state='ak' suffices here. Leave gacc_region=None and set the state variable as shown. 

    31) x1 (Float) - Default = 0.01. The x-position of the signature text box with respect to the axis of the image. 

    32) y1 (Float) - Default = -0.03. The y-position of the signature text box with respect to the axis of the image. 

    33) x2 (Float) - Default = 0.725. The x-position of the timestamp text box with respect to the axis of the image.

    34) y2 (Float) - Default = -0.025. The y-position of the timestamp text box with respect to the axis of the image.

    35) x3 (Float) - Default = 0.01. The x-position of the reference system text box with respect to the axis of the image.

    36) y3 (Float) - Default = 0.01. The y-position of the reference system text box with respect to the axis of the image.     

    37) signature_fontsize (Integer) - Default = 6. The fontsize of the signature. This is only to be changed when making a custom plot. 

    38) stamp_fontsize (Integer) - Default = 5. The fontsize of the timestamp and reference system text. This is only to be changed when making a custom plot. 

    Return: Saves individual images to f:Weather Data/SPC Outlooks/Dry Lightning Outlook/{reference_system}. 
    '''
    PSAs = geometry.get_shapes(f"PSA Shapefiles/National_PSA_Current.shp")
    
    GACC = geometry.get_shapes(f"GACC Boundaries Shapefiles/National_GACC_Current.shp")
    
    CWAs = geometry.get_shapes(f"NWS CWA Boundaries/w_05mr24.shp")
    
    FWZs = geometry.get_shapes(f"NWS Fire Weather Zones/fz05mr24.shp")
    
    PZs = geometry.get_shapes(f"NWS Public Zones/z_05mr24.shp")

    state = state

    if gacc_region != None:
        state = gacc_region
    else:
        state = state

    levels = np.arange(4, 10, 2)

    cmap = colormaps.SPC_Dry_Lightning_Risk_Outlook_colormap()

    reference_system = reference_system


    if reference_system == 'Custom' or reference_system == 'custom':
        show_state_borders = show_state_borders
        show_county_borders = show_county_borders
        show_gacc_borders = show_gacc_borders
        show_psa_borders = show_psa_borders
        show_cwa_borders = show_cwa_borders
        show_nws_firewx_zones = show_nws_firewx_zones
        show_nws_public_zones = show_nws_public_zones

    if reference_system != 'Custom' and reference_system != 'custom':
        
        show_state_borders = False
        show_county_borders = False
        show_gacc_borders = False
        show_psa_borders = False
        show_cwa_borders = False
        show_nws_firewx_zones = False
        show_nws_public_zones = False

        if reference_system == 'States Only':
            show_state_borders = True
        if reference_system == 'States & Counties':
            show_state_borders = True
            show_county_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                county_border_linewidth=0.25
        if reference_system == 'GACC Only':
            show_gacc_borders = True
        if reference_system == 'GACC & PSA':
            show_gacc_borders = True
            show_psa_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.25
        if reference_system == 'CWA Only':
            show_cwa_borders = True
        if reference_system == 'NWS CWAs & NWS Public Zones':
            show_cwa_borders = True
            show_nws_public_zones = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                nws_public_zones_linewidth=0.25
        if reference_system == 'NWS CWAs & NWS Fire Weather Zones':
            show_cwa_borders = True
            show_nws_firewx_zones = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                nws_firewx_zones_linewidth=0.25
        if reference_system == 'NWS CWAs & Counties':
            show_cwa_borders = True
            show_county_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                county_border_linewidth=0.25
        if reference_system == 'GACC & PSA & NWS Fire Weather Zones':
            show_gacc_borders = True
            show_psa_borders = True
            show_nws_firewx_zones = True
            nws_firewx_zones_linewidth=0.25
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.5
        if reference_system == 'GACC & PSA & NWS Public Zones':
            show_gacc_borders = True
            show_psa_borders = True
            show_nws_public_zones = True
            nws_public_zones_linewidth=0.25
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.5
        if reference_system == 'GACC & PSA & NWS CWA':
            show_gacc_borders = True
            show_psa_borders = True
            show_cwa_borders = True
            cwa_border_linewidth=0.25
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.5
        if reference_system == 'GACC & PSA & Counties':
            show_gacc_borders = True
            show_psa_borders = True
            show_county_borders = True
            county_border_linewidth=0.25
        if reference_system == 'GACC & Counties':
            show_gacc_borders = True
            show_county_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                county_border_linewidth=0.25
                
    try:  
        western_bound, eastern_bound, southern_bound, northern_bound, x1, y1, x2, y2, x3, y3, shrink, de, signature_fontsize, stamp_fontsize = settings.get_region_info('NAM', state)
        if state == 'AK' or state == 'ak':
           print(f"This product is only availiable for CONUS.\nSorry OCONUS folks...")
        if state != 'AK' and state != 'ak' or gacc_region != None:
            directory_name = '/SL.us008001/ST.opnl/DF.gr2/DC.ndfd/AR.conus/'

        sp, x, y = settings.get_sp_dims_and_textbox_coords(state)

    except Exception as e:
        western_bound = western_bound
        eastern_bound = eastern_bound
        southern_bound = southern_bound
        northern_bound = northern_bound
        x1=x1
        y1=y1
        x2=x2
        y2=y2
        x3=x3
        y3=y3
        directory_name = '/SL.us008001/ST.opnl/DF.gr2/DC.ndfd/AR.conus/'

    path, path_print = file_functions.noaa_graphics_paths(state, gacc_region, 'Dry Lightning Outlook', reference_system, None, spc=True)

    for file in os.listdir(f"{path}"):
        try:
            os.remove(f"{path}/{file}")
        except Exception as e:
            pass
        
    if data == False:

        ds_short, ds_extended = NDFD_GRIDS.get_ndfd_grids_xarray(directory_name, 'ds.dryfireo.bin', state)

    else:
        ds_short = ds_short
        ds_extended = ds_extended

    short_stop = len(ds_short['step'])
    extended_stop = len(ds_extended['step'])
    extended_start = short_stop + 1

    short_times = ds_short['valid_time']
    extended_times = ds_extended['valid_time']
    short_times = short_times.to_pandas()
    extended_times = extended_times.to_pandas()

    short_start_times, short_end_times, short_start_times_utc = NDFD.get_valid_times_xarray(ds_short, 24)
    extended_start_times, extended_end_times, extended_start_times_utc = NDFD.get_valid_times_xarray(ds_extended, 24)

    init_hr = 12
    hour = short_start_times_utc[0].hour
    dt = False
    if hour == init_hr:
        start = 0
        skip = False
    else:
        if local_time.hour <= 14:
            start = 0
            skip = False
            dt = True
            hours = hour - init_hr
        else:
            start = 1
            skip = True

    if skip == True:
        extended_start = extended_start - 1
    else:
        extended_start = extended_start

    for i in range(start, short_stop, 1):

        fname = f"A_Short_Term_{i}.png"
        if skip == False:
            index = 1 + i
        else:
            index = i

        fig = plt.figure(figsize=(12,12))
        fig.set_facecolor('aliceblue')
        ax = fig.add_subplot(1, 1, 1, projection=mapcrs)
        ax.set_extent([western_bound, eastern_bound, southern_bound, northern_bound], datacrs)
        ax.add_feature(cfeature.COASTLINE.with_scale('50m'), linewidth=0.75, zorder=9)
        ax.add_feature(cfeature.LAND, color='beige', zorder=1)
        ax.add_feature(cfeature.OCEAN, color='lightcyan', zorder=1)
        ax.add_feature(cfeature.LAKES, color='lightcyan', zorder=1)
        ax.add_feature(provinces, linewidth=1, zorder=1)
        
        if show_rivers == True:
            ax.add_feature(cfeature.RIVERS, color='lightcyan', zorder=4)
        else:
            pass
    
        if show_gacc_borders == True:
            ax.add_feature(GACC, linewidth=gacc_border_linewidth, linestyle=gacc_border_linestyle, zorder=6)
        else:
            pass
        if show_psa_borders == True:
            ax.add_feature(PSAs, linewidth=psa_border_linewidth, linestyle=psa_border_linestyle, zorder=5)
        else:
            pass
        if show_county_borders == True:
            ax.add_feature(USCOUNTIES, linewidth=county_border_linewidth, linestyle=county_border_linestyle, zorder=5)
        else:
            pass
        if show_state_borders == True:
            ax.add_feature(cfeature.STATES, linewidth=state_border_linewidth, linestyle=state_border_linestyle, edgecolor='black', zorder=6)
        else:
            pass
        if show_cwa_borders == True:
            ax.add_feature(CWAs, linewidth=cwa_border_linewidth, linestyle=cwa_border_linestyle, zorder=5)
        else:
            pass
        if show_nws_firewx_zones == True:
            ax.add_feature(FWZs, linewidth=nws_firewx_zones_linewidth, linestyle=nws_firewx_zones_linestyle, zorder=5)
        else:
            pass
        if show_nws_public_zones == True:
            ax.add_feature(PZs, linewidth=nws_public_zones_linewidth, linestyle=nws_public_zones_linestyle, zorder=5)
        else:
            pass  

        start = short_start_times[i]
        end = short_end_times[i]             

        cs = ax.contourf(ds_short['longitude'][:, :], ds_short['latitude'][:, :], ds_short['unknown'][i, :, :], levels=levels, cmap=cmap, transform=datacrs, alpha=0.5)
        cbar = fig.colorbar(cs, shrink=shrink, pad=0.01, location='right', ticks=[5, 7])
        cbar.ax.set_yticklabels(['Isolated', 'Scattered'])

        plt.title(f"SPC Outlooks [Day {index}]\nDry Lightning Outlook", fontsize=8, fontweight='bold', loc='left')
        if dt == False:
            plt.title(f"Valid: {start.strftime(f'%a %b %d')}", fontsize=7, fontweight='bold', loc='right')
        else:
            start_hour = short_start_times[0].hour - hours
            end_hour = short_end_times[0].hour - hours
            plt.title(f"Valid: {start.strftime(f'%a %b %d')}", fontsize=7, fontweight='bold', loc='right')

        ax.text(x1, y1, "Plot Created With FireWxPy (C) Eric J. Drewitz " +utc_time.strftime('%Y')+" | Data Source: NOAA/NCEP/SPC", transform=ax.transAxes, fontsize=signature_fontsize, fontweight='bold', bbox=props)
        ax.text(x2, y2, "Image Created: " + local_time.strftime(f'%m/%d/%Y %H:%M {timezone}') + " (" + utc_time.strftime('%H:%M UTC') + ")", transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props)
        ax.text(x3, y3, "Reference System: "+reference_system, transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props, zorder=11)  

        fig.savefig(f"{path}/{fname}", bbox_inches='tight')
    print(f"Saved short-term forecast graphics to {path_print}")


    for i in range(0, extended_stop, 1):

        fname = f"B_Extended_Term_{i}.png"

        index = extended_start + i

        fig = plt.figure(figsize=(12,12))
        fig.set_facecolor('aliceblue')
        ax = fig.add_subplot(1, 1, 1, projection=mapcrs)
        ax.set_extent([western_bound, eastern_bound, southern_bound, northern_bound], datacrs)
        ax.add_feature(cfeature.COASTLINE.with_scale('50m'), linewidth=0.75, zorder=9)
        ax.add_feature(cfeature.LAND, color='beige', zorder=1)
        ax.add_feature(cfeature.OCEAN, color='lightcyan', zorder=1)
        ax.add_feature(cfeature.LAKES, color='lightcyan', zorder=1)
        ax.add_feature(provinces, linewidth=1, zorder=1)
        if show_rivers == True:
            ax.add_feature(cfeature.RIVERS, color='lightcyan', zorder=4)
        else:
            pass
    
        if show_gacc_borders == True:
            ax.add_feature(GACC, linewidth=gacc_border_linewidth, linestyle=gacc_border_linestyle, zorder=6)
        else:
            pass
        if show_psa_borders == True:
            ax.add_feature(PSAs, linewidth=psa_border_linewidth, linestyle=psa_border_linestyle, zorder=5)
        else:
            pass
        if show_county_borders == True:
            ax.add_feature(USCOUNTIES, linewidth=county_border_linewidth, linestyle=county_border_linestyle, zorder=5)
        else:
            pass
        if show_state_borders == True:
            ax.add_feature(cfeature.STATES, linewidth=state_border_linewidth, linestyle=state_border_linestyle, edgecolor='black', zorder=6)
        else:
            pass
        if show_cwa_borders == True:
            ax.add_feature(CWAs, linewidth=cwa_border_linewidth, linestyle=cwa_border_linestyle, zorder=5)
        else:
            pass
        if show_nws_firewx_zones == True:
            ax.add_feature(FWZs, linewidth=nws_firewx_zones_linewidth, linestyle=nws_firewx_zones_linestyle, zorder=5)
        else:
            pass
        if show_nws_public_zones == True:
            ax.add_feature(PZs, linewidth=nws_public_zones_linewidth, linestyle=nws_public_zones_linestyle, zorder=5)
        else:
            pass  

        start = extended_start_times[i]
        end = extended_end_times[i]

        cs = ax.contourf(ds_extended['longitude'][:, :], ds_extended['latitude'][:, :], ds_extended['unknown'][i, :, :], levels=levels, cmap=cmap, transform=datacrs, alpha=0.5)

        cbar = fig.colorbar(cs, shrink=shrink, pad=0.01, location='right', ticks=[5, 7])
        cbar.ax.set_yticklabels(['Isolated', 'Scattered'])

        plt.title(f"SPC Outlooks [Day {index}]\nDry Lightning Outlook", fontsize=8, fontweight='bold', loc='left')
        plt.title(f"Valid: {start.strftime(f'%a %b %d')}", fontsize=7, fontweight='bold', loc='right')

        ax.text(x1, y1, "Plot Created With FireWxPy (C) Eric J. Drewitz " +utc_time.strftime('%Y')+" | Data Source: NOAA/NCEP/SPC", transform=ax.transAxes, fontsize=signature_fontsize, fontweight='bold', bbox=props)
        ax.text(x2, y2, "Image Created: " + local_time.strftime(f'%m/%d/%Y %H:%M {timezone}') + " (" + utc_time.strftime('%H:%M UTC') + ")", transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props)
        ax.text(x3, y3, "Reference System: "+reference_system, transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props, zorder=11)   

        fig.savefig(f"{path}/{fname}", bbox_inches='tight')
    print(f"Saved extended forecast graphics to {path_print}")


def plot_convective_outlook(western_bound=None, eastern_bound=None, southern_bound=None, northern_bound=None, shrink=0.7, show_rivers=True, reference_system='States & Counties', show_state_borders=False, show_county_borders=False, show_gacc_borders=False, show_psa_borders=False, show_cwa_borders=False, show_nws_firewx_zones=False, show_nws_public_zones=False, state_border_linewidth=1, county_border_linewidth=0.25, gacc_border_linewidth=1, psa_border_linewidth=1, cwa_border_linewidth=1, nws_firewx_zones_linewidth=0.5, nws_public_zones_linewidth=0.5, state_border_linestyle='-', county_border_linestyle='-', gacc_border_linestyle='-', psa_border_linestyle='-', cwa_border_linestyle='-', nws_firewx_zones_linestyle='-', nws_public_zones_linestyle='-', data=False, ds=None, state='conus', gacc_region=None, x1=0.01, y1=-0.03, x2=0.725, y2=-0.025, x3=0.01, y3=0.01, signature_fontsize=6, stamp_fontsize=5):


    r'''
    This function plots the latest available NOAA/NWS SPC Convective Outlook. 

    Required Arguments: None

    Optional Arguments: 

    1) western_bound (Integer or Float) - Default = None. Western extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Negative values denote the western hemisphere and positive 
       values denote the eastern hemisphere. 

    2) eastern_bound (Integer or Float) - Default = None. Eastern extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Negative values denote the western hemisphere and positive 
       values denote the eastern hemisphere. 

    3) southern_bound (Integer or Float) - Default = None. Southern extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Positive values denote the northern hemisphere and negative 
       values denote the southern hemisphere. 

    4) northern_bound (Integer or Float) - Default = None. Northern extent of the plot in decimal degrees. 
       The default setting is None. If set to None, the user must select a state or gacc_region. 
       This setting should be changed from None to an integer or float value if the user wishes to
       have a custom area selected. Positive values denote the northern hemisphere and negative 
       values denote the southern hemisphere.
    
    5) shrink (Integer or Float) - Default = 0.7. This is how the colorbar is sized to the figure. 
       This is a feature of matplotlib, as per their definition, the shrink is:
       "Fraction by which to multiply the size of the colorbar." 
       This should only be changed if the user wishes to change the size of the colorbar. 
       Preset values are called from the settings module for each state and/or gacc_region.

    6) show_rivers (Boolean) - If set to True, rivers will display. If set to False, county borders will not display. 
        Default setting is True. Users should change this value to False if they wish to hide rivers.

    7) reference_system (String) - Default = 'States & Counties'. The georgraphical reference system with respect to the borders on the map. If the user
        wishes to use a reference system not on this list, please see items 17-23. 
        Reference Systems: 
        
        1) 'States & Counties'
        2) 'States Only'
        3) 'GACC Only'
        4) 'GACC & PSA'
        5) 'CWA Only'
        6) 'NWS CWAs & NWS Public Zones'
        7) 'NWS CWAs & NWS Fire Weather Zones'
        8) 'NWS CWAs & Counties'
        9) 'GACC & PSA & NWS Fire Weather Zones'
        10) 'GACC & PSA & NWS Public Zones'
        11) 'GACC & PSA & NWS CWA'
        12) 'GACC & PSA & Counties'
        13) 'GACC & Counties'
                           

    8) show_state_borders (Boolean) - If set to True, state borders will display. If set to False, state borders will not display. 
        Default setting is False. Users should change this value to False if they wish to hide state borders. 

    9) show_county_borders (Boolean) - If set to True, county borders will display. If set to False, county borders will not display. 
        Default setting is False. Users should change this value to False if they wish to hide county borders. 

    10) show_gacc_borders (Boolean) - If set to True, GACC (Geographic Area Coordination Center) borders will display. If set to False, GACC borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display GACC borders. 

    11) show_psa_borders (Boolean) - If set to True, PSA (Predictive Services Area) borders will display. If set to False, PSA borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display PSA borders.

    12) show_cwa_borders (Boolean) - If set to True, CWA borders will display. If set to False, CWA borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display CWA borders.

    13) show_nws_firewx_zones (Boolean) - If set to True, NWS FWZ borders will display. If set to False, NWS FWZ borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display NWS FWZ borders.

    14) show_nws_public_zones (Boolean) - If set to True, NWS Public Zone borders will display. If set to False, NWS Public Zone borders will not display. 
        Default setting is False. Users should change this value to True if they wish to display NWS Public Zone borders.

    15) state_border_linewidth (Integer) - Linewidth (thickness) of the state borders. Default setting is 2. 

    16) county_border_linewidth (Integer) - Linewidth (thickness) of the county borders. Default setting is 1. 

    17) gacc_border_linewidth (Integer) - Linewidth (thickness) of the GACC borders. Default setting is 2. 

    18) psa_border_linewidth (Integer) - Linewidth (thickness) of the PSA borders. Default setting is 1. 

    19) state_border_linestyle (String) - Linestyle of the state borders. Default is a solid line. 
        To change to a dashed line, users should set state_border_linestyle='--'. 

    20) county_border_linestyle (String) - Linestyle of the county borders. Default is a solid line. 
        To change to a dashed line, users should set county_border_linestyle='--'. 

    21) gacc_border_linestyle (String) - Linestyle of the GACC borders. Default is a solid line. 
        To change to a dashed line, users should set gacc_border_linestyle='--'. 

    22) psa_border_linestyle (String) - Linestyle of the PSA borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    23) cwa_border_linestyle (String) - Linestyle of the CWA borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    24) nws_firewx_zones_linestyle (String) - Linestyle of the NWS FWZ borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    25) nws_public_zones_linestyle (String) - Linestyle of the NWS Public Zone borders. Default is a solid line. 
        To change to a dashed line, users should set psa_border_linestyle='--'. 

    26) data (Boolean) - Default = False. This tells the function if the user is downloading the data outside of the function
        and passing the data in or if the function needs to download the data. A value of False means the data
        is downloaded inside of the function while a value of True means the user is downloading the data outside
        of the function and passing it in. For users who intend to make a lot of graphics for a lot of different 
        things, it is recommended to set this value to True and download the data outside of the function and pass
        it in so that the amount of data requests on the host servers can be minimized. 

    27) ds (xarray.dataarray) - The xarray data array of the downloaded NWS Short-Term Forecast dataset from the NWS FTP Server. 
        This setting is only to be changed if the user wants to limit the amount of downloads from the 
        NWS FTP Server. Default setting is None. Please see the documentation for the data_access module 
        if the user wishes to download the data outside of this function. 

    28) state (String) - The two letter state abbreviation for the state the user wishes to make the graphic for. 
        If the user wishes to make a graphic for the entire CONUS, there are 2 acceptable abbreviations: 'CONUS' or 'conus'. 
        Example: If the user wishes to make a plot for the state of California both 'CA' or 'ca' are
        acceptable. Default setting is 'conus'. If the user wishes to make a plot based on gacc_region, this value must be 
        changed to None. 

    29) gacc_region (String) - The abbreviation for each of the 10 GACC regions. Default setting is None. 
        If the user wishes to make a plot based on GACC Region than state, the state variable must be set to 
        None and the gacc_region variable must be set to one of the acceptable abbreviations. 

        Here is a list of acceptable gacc_region abbreviations:

        South Ops: 'OSCC' or 'oscc' or 'SOPS' or 'sops'
        
        North Ops: 'ONCC' or 'oncc' or 'NOPS' or 'nops'
        
        Great Basin: 'GBCC' or 'gbcc' or 'GB' or 'gb'
        
        Northern Rockies: 'NRCC' or 'nrcc' or 'NR' or 'nr'
        
        Rocky Mountain: 'RMCC' or 'rmcc' or 'RM' or 'rm'
        
        Southwest: 'SWCC' or 'swcc' or 'SW' or 'sw'
        
        Southern: 'SACC' or 'sacc' or 'SE' or 'se'
        
        Eastern: 'EACC' or 'eacc' or 'E' or 'e'
        
        Pacific Northwest: 'PNW' or 'pnw' or 'NWCC' or 'nwcc' or 'NW' or 'nw'
        
        Alaska: Setting state='AK' or state='ak' suffices here. Leave gacc_region=None and set the state variable as shown. 

    30) x1 (Float) - Default = 0.01. The x-position of the signature text box with respect to the axis of the image. 

    31) y1 (Float) - Default = -0.03. The y-position of the signature text box with respect to the axis of the image. 

    32) x2 (Float) - Default = 0.725. The x-position of the timestamp text box with respect to the axis of the image.

    33) y2 (Float) - Default = -0.025. The y-position of the timestamp text box with respect to the axis of the image.

    34) x3 (Float) - Default = 0.01. The x-position of the reference system text box with respect to the axis of the image.

    35) y3 (Float) - Default = 0.01. The y-position of the reference system text box with respect to the axis of the image.     

    36) signature_fontsize (Integer) - Default = 6. The fontsize of the signature. This is only to be changed when making a custom plot. 

    37) stamp_fontsize (Integer) - Default = 5. The fontsize of the timestamp and reference system text. This is only to be changed when making a custom plot. 

    Return: Saves individual images to f:Weather Data/SPC Outlooks/Convective Outlook/{reference_system}. 
    '''
    PSAs = geometry.get_shapes(f"PSA Shapefiles/National_PSA_Current.shp")
    
    GACC = geometry.get_shapes(f"GACC Boundaries Shapefiles/National_GACC_Current.shp")
    
    CWAs = geometry.get_shapes(f"NWS CWA Boundaries/w_05mr24.shp")
    
    FWZs = geometry.get_shapes(f"NWS Fire Weather Zones/fz05mr24.shp")
    
    PZs = geometry.get_shapes(f"NWS Public Zones/z_05mr24.shp")

    state = state

    if gacc_region != None:
        state = gacc_region
    else:
        state = state

    levels = [1, 2, 3, 4, 5, 6, 8]

    cmap = colormaps.spc_convective_outlook_colormap()

    reference_system = reference_system


    if reference_system == 'Custom' or reference_system == 'custom':
        show_state_borders = show_state_borders
        show_county_borders = show_county_borders
        show_gacc_borders = show_gacc_borders
        show_psa_borders = show_psa_borders
        show_cwa_borders = show_cwa_borders
        show_nws_firewx_zones = show_nws_firewx_zones
        show_nws_public_zones = show_nws_public_zones

    if reference_system != 'Custom' and reference_system != 'custom':
        
        show_state_borders = False
        show_county_borders = False
        show_gacc_borders = False
        show_psa_borders = False
        show_cwa_borders = False
        show_nws_firewx_zones = False
        show_nws_public_zones = False

        if reference_system == 'States Only':
            show_state_borders = True
        if reference_system == 'States & Counties':
            show_state_borders = True
            show_county_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                county_border_linewidth=0.25
        if reference_system == 'GACC Only':
            show_gacc_borders = True
        if reference_system == 'GACC & PSA':
            show_gacc_borders = True
            show_psa_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.25
        if reference_system == 'CWA Only':
            show_cwa_borders = True
        if reference_system == 'NWS CWAs & NWS Public Zones':
            show_cwa_borders = True
            show_nws_public_zones = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                nws_public_zones_linewidth=0.25
        if reference_system == 'NWS CWAs & NWS Fire Weather Zones':
            show_cwa_borders = True
            show_nws_firewx_zones = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                nws_firewx_zones_linewidth=0.25
        if reference_system == 'NWS CWAs & Counties':
            show_cwa_borders = True
            show_county_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                county_border_linewidth=0.25
        if reference_system == 'GACC & PSA & NWS Fire Weather Zones':
            show_gacc_borders = True
            show_psa_borders = True
            show_nws_firewx_zones = True
            nws_firewx_zones_linewidth=0.25
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.5
        if reference_system == 'GACC & PSA & NWS Public Zones':
            show_gacc_borders = True
            show_psa_borders = True
            show_nws_public_zones = True
            nws_public_zones_linewidth=0.25
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.5
        if reference_system == 'GACC & PSA & NWS CWA':
            show_gacc_borders = True
            show_psa_borders = True
            show_cwa_borders = True
            cwa_border_linewidth=0.25
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                psa_border_linewidth=0.5
        if reference_system == 'GACC & PSA & Counties':
            show_gacc_borders = True
            show_psa_borders = True
            show_county_borders = True
            county_border_linewidth=0.25
        if reference_system == 'GACC & Counties':
            show_gacc_borders = True
            show_county_borders = True
            if state == 'US' or state == 'us' or state == 'USA' or state == 'usa':
                county_border_linewidth=0.25
                
    try:  
        western_bound, eastern_bound, southern_bound, northern_bound, x1, y1, x2, y2, x3, y3, shrink, de, signature_fontsize, stamp_fontsize = settings.get_region_info('NAM', state)
        if state == 'AK' or state == 'ak':
           print(f"This product is only availiable for CONUS.\nSorry OCONUS folks...")
        if state != 'AK' and state != 'ak' or gacc_region != None:
            directory_name = '/SL.us008001/ST.opnl/DF.gr2/DC.ndfd/AR.conus/'

        sp, x, y = settings.get_sp_dims_and_textbox_coords(state)

    except Exception as e:
        western_bound = western_bound
        eastern_bound = eastern_bound
        southern_bound = southern_bound
        northern_bound = northern_bound
        x1=x1
        y1=y1
        x2=x2
        y2=y2
        x3=x3
        y3=y3
        directory_name = '/SL.us008001/ST.opnl/DF.gr2/DC.ndfd/AR.conus/'

    path, path_print = file_functions.noaa_graphics_paths(state, gacc_region, 'Convective Outlook', reference_system, None, spc=True)

    for file in os.listdir(f"{path}"):
        try:
            os.remove(f"{path}/{file}")
        except Exception as e:
            pass
        
    if data == False:

        ds_short, ds_extended = NDFD_GRIDS.get_ndfd_grids_xarray(directory_name, 'ds.conhazo.bin', state)

    else:
        ds_short = ds

    short_stop = len(ds_short['step'])

    short_times = ds_short['valid_time']
    short_times = short_times.to_pandas()

    short_start_times, short_end_times, short_start_times_utc = NDFD.get_valid_times_xarray(ds_short, 24)

    init_hr = 12
    hour = short_start_times_utc[0].hour
    dt = False
    if hour == init_hr:
        start = 0
        skip = False
    else:
        if local_time.hour <= 14:
            start = 0
            skip = False
            dt = True
            hours = hour - init_hr
        else:
            start = 1
            skip = True

    for i in range(start, short_stop, 1):

        fname = f"A_Short_Term_{i}.png"
        if skip == False:
            index = 1 + i
        else:
            index = i

        fig = plt.figure(figsize=(12,12))
        fig.set_facecolor('aliceblue')
        ax = fig.add_subplot(1, 1, 1, projection=mapcrs)
        ax.set_extent([western_bound, eastern_bound, southern_bound, northern_bound], datacrs)
        ax.add_feature(cfeature.COASTLINE.with_scale('50m'), linewidth=0.75, zorder=9)
        ax.add_feature(cfeature.LAND, color='beige', zorder=1)
        ax.add_feature(cfeature.OCEAN, color='lightcyan', zorder=1)
        ax.add_feature(cfeature.LAKES, color='lightcyan', zorder=1)
        ax.add_feature(provinces, linewidth=1, zorder=1)
        
        if show_rivers == True:
            ax.add_feature(cfeature.RIVERS, color='lightcyan', zorder=4)
        else:
            pass
    
        if show_gacc_borders == True:
            ax.add_feature(GACC, linewidth=gacc_border_linewidth, linestyle=gacc_border_linestyle, zorder=6)
        else:
            pass
        if show_psa_borders == True:
            ax.add_feature(PSAs, linewidth=psa_border_linewidth, linestyle=psa_border_linestyle, zorder=5)
        else:
            pass
        if show_county_borders == True:
            ax.add_feature(USCOUNTIES, linewidth=county_border_linewidth, linestyle=county_border_linestyle, zorder=5)
        else:
            pass
        if show_state_borders == True:
            ax.add_feature(cfeature.STATES, linewidth=state_border_linewidth, linestyle=state_border_linestyle, edgecolor='black', zorder=6)
        else:
            pass
        if show_cwa_borders == True:
            ax.add_feature(CWAs, linewidth=cwa_border_linewidth, linestyle=cwa_border_linestyle, zorder=5)
        else:
            pass
        if show_nws_firewx_zones == True:
            ax.add_feature(FWZs, linewidth=nws_firewx_zones_linewidth, linestyle=nws_firewx_zones_linestyle, zorder=5)
        else:
            pass
        if show_nws_public_zones == True:
            ax.add_feature(PZs, linewidth=nws_public_zones_linewidth, linestyle=nws_public_zones_linestyle, zorder=5)
        else:
            pass  

        start = short_start_times[i]
        end = short_end_times[i]             

        cs = ax.contourf(ds_short['longitude'][:, :], ds_short['latitude'][:, :], ds_short['unknown'][i, :, :], levels=levels, cmap=cmap, transform=datacrs, alpha=0.5)
        cbar = fig.colorbar(cs, shrink=shrink, pad=0.01, location='right', ticks=[2, 3, 4, 5, 6, 8])
        cbar.ax.set_yticklabels(['Thunderstorms', 'Marginal Risk', 'Slight Risk', 'Enhanced Risk', 'Moderate Risk', 'High Risk'])

        plt.title(f"SPC Outlooks [Day {index}]\nConvective Outlook", fontsize=8, fontweight='bold', loc='left')
        if dt == False:
            plt.title(f"Valid: {start.strftime(f'%a %b %d')}", fontsize=7, fontweight='bold', loc='right')
        else:
            start_hour = short_start_times[0].hour - hours
            end_hour = short_end_times[0].hour - hours
            plt.title(f"Valid: {start.strftime(f'%a %b %d')}", fontsize=7, fontweight='bold', loc='right')

        ax.text(x1, y1, "Plot Created With FireWxPy (C) Eric J. Drewitz " +utc_time.strftime('%Y')+" | Data Source: NOAA/NCEP/SPC", transform=ax.transAxes, fontsize=signature_fontsize, fontweight='bold', bbox=props)
        ax.text(x2, y2, "Image Created: " + local_time.strftime(f'%m/%d/%Y %H:%M {timezone}') + " (" + utc_time.strftime('%H:%M UTC') + ")", transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props)
        ax.text(x3, y3, "Reference System: "+reference_system, transform=ax.transAxes, fontsize=stamp_fontsize, fontweight='bold', bbox=props, zorder=11)  

        fig.savefig(f"{path}/{fname}", bbox_inches='tight')
    print(f"Saved forecast graphics to {path_print}")












"""
This file is written by (C) Meteorologist Eric J. Drewitz
                                     USDA/USFS

"""

import pandas as pd
import warnings
warnings.filterwarnings('ignore')

from dateutil import tz
from datetime import datetime, timedelta
from firewxpy.calc import unit_conversion

from_zone = tz.tzutc()
to_zone = tz.tzlocal()

class NDFD:

    r'''
    This class hosts functions to parse the NDFD GRIB2 files

    '''

    def ndfd_to_dataframe(ds, parameter, diff=False, temperature_to_F=False, decimate=None):

        r'''
        This function parses the NDFD GRIB2 file and converts the data array into a pandas dataframe

        Required Arguments:

        1) ds (xarray.dataarray) - The NDFD GRIB2 dataset

        2) parameter (String) - The variable name. 

        Optional Arguments:

        1) diff (Boolean) - Default = False. If set to True, the difference between value[i+1] and value[i] will be returned (val[i+1] - val[i]). 

        2) temperature_to_F (Boolean) - Default = False. If set to True, values will be converted from Kelvin to Fahrenheit.

        3) decimate (Integer) - Default = None. If set to a non-None value, the data will be decimated on both the x and y coordinates by the magnitude of the set value.

        Return: A pandas dataframe of the NDFD values. 

        '''

        ds = ds

        try:
            stop = len(ds['valid_time'])
        except Exception as e:
            stop = len(ds['step'])

        vals = []

        if temperature_to_F == True:
            ds[parameter] = unit_conversion.Temperature_Data_or_Dewpoint_Data_Kelvin_to_Fahrenheit(ds[parameter])
        else:
            pass

        if decimate != None:
            if diff == False:
                for i in range(0, stop, 1):
                    val = ds[parameter][i, ::decimate, ::decimate].to_dataframe()
                    vals.append(val) 
            else:
                for i in range(1, stop, 1):
                    try:
                        val = ds[parameter][i, ::decimate, ::decimate] - ds[parameter][i-1, ::decimate, ::decimate]
                        val = val.to_dataframe()
                        vals.append(val)    
                    except Exception as e:
                        pass                
            
        if decimate == None:

            if diff == False:
                for i in range(0, stop, 1):
                    val = ds[parameter][i, :, :].to_dataframe()
                    vals.append(val) 
    
            else:
                for i in range(1, stop, 1):
                    try:
                        val = ds[parameter][i, :, :] - ds[parameter][i-1, :, :]
                        val = val.to_dataframe()
                        vals.append(val)    
                    except Exception as e:
                        pass

        return vals
        

    def get_valid_times_xarray(ds, time_interval):

        r'''
        This function will get all of the forecast validity times for the time length of the grid for the specific weather element in NDFD. 

        Required Arguments:

        1) ds (xarray.dataarray) - The NDFD GRIB2 dataset

        2) time_interval (Integer) - The time length of the grid in hours. 

        Returns: A list of the start and end times in local time for each forecast interval. 

        '''

        ds = ds

        stop = len(ds['valid_time'])

        start_times = []
        end_times = []
        start_times_utc = []

        for i in range(0, stop, 1):
            end_time = ds['valid_time'][i]
            end_time = end_time.to_pandas()
            end_time = pd.to_datetime(end_time)
            end_time = end_time.replace(tzinfo=from_zone)
            end_time = end_time.astimezone(to_zone)
            end_times.append(end_time)

        for i in range(0, stop, 1):
            start_time = end_times[i] - timedelta(hours=time_interval)
            start_times.append(start_time)
            start_time_utc = start_time.replace(tzinfo=to_zone)
            start_time_utc = start_time.astimezone(from_zone)
            start_times_utc.append(start_time_utc)

        return start_times, end_times, start_times_utc

class checks:

    r'''

    This class hosts functions to check various things:

    1) Wind Direction
    2) RTMA vs. METAR Observation times

    '''

    def wind_direction_number_to_abbreviation(wind_direction):

        r'''
        This function takes the numerical wind direction and assigns an abbreviation (i.e. N vs. NW) to the value

        Inputs:
                1) wind_direction (Integer or Float)

        Returns:
                1) wind_direction (String)

        '''
        wind_direction = wind_direction
        
        if wind_direction >= 358 or wind_direction <= 2:
            wind_dir = 'N'
        if wind_direction > 2 and wind_direction <= 30:
            wind_dir = 'NNE'
        if wind_direction > 30 and wind_direction <= 60:
            wind_dir = 'NE'
        if wind_direction > 60 and wind_direction < 88:
            wind_dir = 'ENE'
        if wind_direction >= 88 and wind_direction <= 92:
            wind_dir = 'E'
        if wind_direction > 92 and wind_direction <= 120:
            wind_dir = 'ESE'
        if wind_direction > 120 and wind_direction <= 150:
            wind_dir = 'SE'
        if wind_direction > 150 and wind_direction < 178:
            wind_dir = 'SSE'
        if wind_direction >= 178 and wind_direction <= 182:
            wind_dir = 'S'
        if wind_direction > 182 and wind_direction <= 210:
            wind_dir = 'SSW'
        if wind_direction > 210 and wind_direction <= 240:
            wind_dir = 'SW'
        if wind_direction > 240 and wind_direction < 268:
            wind_dir = 'WSW'
        if wind_direction >= 268 and wind_direction <= 272:
            wind_dir = 'W'
        if wind_direction > 272 and wind_direction <= 300:
            wind_dir = 'WNW'
        if wind_direction > 300 and wind_direction <= 330:
            wind_dir = 'NW'
        if wind_direction > 330 and wind_direction < 358:
            wind_dir = 'NNW'

        return wind_dir
    

    def check_RTMA_vs_METAR_Times(real_time_mesoscale_analysis_time, metar_observation_time):

        r'''
        This function compares the time of the RTMA and the time of the observations to make them match. 

        Required Arguments: 

        1) real_time_mesoscale_analysis_time (datetime) - The time of the RTMA

        2) metar_observation_time (datetime) - The time of the METAR observations

        Returns: The latest time where there is a match between RTMA & METAR Observations

        '''

        metar_time = metar_observation_time

        rtma_time = real_time_mesoscale_analysis_time

        time_diff = metar_time.hour - rtma_time.hour

        if metar_time.hour > rtma_time.hour:
            new_metar_time = metar_time - timedelta(hours=time_diff)

        if metar_time.hour < rtma_time.hour:
            hour = rtma_time.hour
            new_metar_time = metar_time - timedelta(days=1)
            year = new_metar_time.year
            month = new_metar_time.month
            day = new_metar_time.day
            new_metar_time = datetime(year, month, day, hour)

        else:
            new_metar_time = rtma_time
            

        return new_metar_time






        
